/***************************************************************************
 *   Copyright (C) 2005 by Sebastian Beck                                  *
 *   sebastian.bw@freenet.de                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qpainter.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qtextedit.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qstylesheet.h>
#include <qregion.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qfile.h>
#include <qdir.h>
#include <qevent.h>
#include <qdatastream.h>
#include <qurloperator.h>

#include <ktrader.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include "khistorydoublenuminput.h"
#include <kfiledialog.h>
#include <kmainwindow.h>
#include <kdesktopfile.h>
#include <kpassdlg.h>
#include <kstatusbar.h>
#include <kapplication.h>
#include <kprocess.h>

#include <math.h>

#include "getcircle.h"
#include "bohrung.h"
#include "bohrloch_fasen.h"
#include "bohrloch_versenken.h"
#include "topfloch.h"
#include "zylinder.h"
#include "tasche.h"
#include "quader.h"
#include "werkzeugwechsel.h"
#include "awwahl.h"
#include "konturdialog.h"
#include "wechselpos.h"
#include "wwechsel.h"
#include "wwechsel1.h"
#include "maschinenauswahl.h"

#include "pcncview.h"

	uchar bit[7];
	unsigned char c = 0;		//  8 Bit
	unsigned short int si;			// 16 Bit
	unsigned long int l=0, schleif=0;	// 32 Bit
	

PCNCView::PCNCView(QWidget *parent): M3Dansicht(parent)
	{
	mfaktor = 2;
	n = 0;
	m = 0;
	f = 0;
	X = 0;
	Y = 0;
	Z = 0;
	OK = TRUE;
	FIRST = TRUE;
	sidebar = 52;
	min_timesteps = 500;
	aM = -1;
	aW = -1;
	line = "";
	dir = QDir::homeDirPath()+QString("/PCNC");
	steuerprog = "pcnc-plus";
	nc_datei = "/default.nc";
	werkzeug = "";
	QDir::setCurrent( dir );
	
	Bo = new Bohrung(this); Bo->hide();Bo->move( 54, 4);
	Bf = new Bohrloch_fasen(this); Bf->hide();Bf->move( 54, 4);
	Bv = new Bohrloch_versenken(this); Bv->hide();Bv->move( 54, 4);
	To = new Topfloch(this); To->hide();To->move( 54, 4);
	Zy = new Zylinder(this); Zy->hide();Zy->move( 54, 4);
	Ta = new Tasche(this); Ta->hide();Ta->move( 54, 4);
	Qu = new Quader(this); Qu->hide();Qu->move( 54, 4);
	Tx = new Text(this); Tx->hide();Tx->move( 54, 4);
	
	w = new WWechsel(this);
	w1 = new WWechsel1(this);
	AWw = new AWwahl(this);
	ma = new MaschinenAuswahl(this);
	Li = new Linie(this); Li->hide();
	Lix = new Liniex(this); Lix->hide();
	Liy = new Liniey(this); Liy->hide();
	Liz = new Liniez(this); Liz->hide();
	Kr = new Kreis(this); Kr->hide();
	Vrs = new Vorschub(this); Vrs->hide();
	S = new Schleife(this); S->hide();
	Sp = new Spiel(this); Sp->hide();
	Ww = new Werkzeugwechsel(this); Ww->hide();
	El = new Ellipse(this); El->hide();
	M = new Memo(this); M->hide();
	B = new Beschleunigung(this); B->hide();
	
	FL = new F_Linear(this); FL->hide();
	FK = new F_Kreis(this); FK->hide();
	FE = new F_Ellips(this); FE->hide();
	
	lastwidget_left = Bo;
	lastwidget_right = Li;
	lastwidget_F = FL;
	
	ma->sdat = "pcnc-plus";
	
	wtool = new KProcess;
	wtool->setUseShell( TRUE );
	if( 0 == getuid() )
		*wtool << "werkzeugtool";
	else
		*wtool << "kdesu -c 'werkzeugtool'";
	
	mtool = new KProcess;
	mtool->setUseShell( TRUE );
	if( 0 == getuid() )
		*mtool << "maschinentool";
	else
		*mtool << "kdesu -c 'maschinentool'";
	
	steur = new KProcess;
	steur->setUseShell( TRUE );
	
	ncbefehlseditor->setOverwriteMode ( FALSE );
	this->resize( 600, 480 );
	}

PCNCView::~PCNCView()
	{
	}

QWidget* PCNCView::showclicked( QWidget *n, QWidget *lastwidget )
	{
	if( lastwidget->isVisible() )
		{
		if( lastwidget == n )
			{
			lastwidget->hide();
			return n;
			}
		
		lastwidget->hide();
		n->show();
		}
	else
		n->show();
	
	return n;
	}

void PCNCView::quit()
	{	
	if( ncbefehlseditor->isModified() )
		if( KMessageBox::Yes == KMessageBox::questionYesNo( 0, QString("Die Datei %1 wurde verndert !\nSpeichern ?").arg(nc_datei), "Achtung") )
			if( nc_datei == "Werkstcke/default.nc" )
				{
				PCNCView::writeto();
				}
			else
				{
				write();
				}
	exit(0);
	}

int PCNCView::fitsin( double big, double small, double *rest=NULL )
	{
	int count, b, s;
	
	b = int( round( big * 100 ) );
	s = int( round( small * 100 ) );
	count = 0;
	while( b >= s )
		{
		b -= s;
		count ++;
		}
	
	*rest = b / 100.00;
	return count;
	}


#include "getellips.inc"
#include "getcurrentpos.inc"
#include "getstringnum.inc"


bool PCNCView::defAW( int *tool )
	{
	AWw->DELETED = FALSE;
	AWw->werkzeuge->clear();
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) )
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		while ( ! str1.atEnd()  )
			{
			str1 >> name;
			AWw->werkzeuge->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		f1.close();
		}
	AWw->werkzeuge->insertItem( "Bitte atuelles Werkzeug angeben", -1 );
	AWw->werkzeuge->setCurrentItem( AWw->werkzeuge->count() - 1 );
	
	AWw->exec();
	
	if( AWw->DELETED )
		*tool = AWw->werkzeuge->currentItem();
		
	return AWw->DELETED;
	}

void PCNCView::firstTool( int i )
	{
	char l = 54;
	aW = i;
	while( i > 9 )
		{
		i /= 10;
		l ++;
		}
	
	line = "MEMO\n lnge	"+QString::number( l )+"\nKontrollieren Sie, da Werkzeug Nr. "+QString::number( aW )+" eingespannt ist.\nWARTE\n";
	ncbefehlseditor->insert( line );
	FIRST = FALSE;
	}


void PCNCView::resizeEvent( QResizeEvent * e )
	{
	Li->move( e->size().width() - 292-sidebar, 2 );
	Lix->move( e->size().width() - 292-sidebar, 50 );
	Liy->move( e->size().width() - 292-sidebar, 98 );
	Liz->move( e->size().width() - 292-sidebar, 146 );
	Kr->move( e->size().width() - 292-sidebar, 194 );
	Vrs->move( e->size().width() - 292-sidebar, 242 );
	Sp->move( e->size().width() - 292-sidebar, 386 );

// Die Zahl in if = Widgethhe + Position_oben(48*x +2) + 3,   unter else ist die Position_oben(48*x +2),   nach if ist somit Widgethhe + 3

	if( e->size().height() < 586 )
		S->move( e->size().width() - 292-sidebar, e->size().height()-248 );
	else
		S->move( e->size().width() - 292-sidebar, 338 );

	if( e->size().height() < 623)
		Ww->move( e->size().width() - 292-sidebar, e->size().height()-333 );
	else
		Ww->move( e->size().width() - 292-sidebar, 290 );
	
	if( e->size().height() < 713)
		M->move( e->size().width() - 292-sidebar, e->size().height()-183 );
	else
		M->move( e->size().width() - 292-sidebar, 530 );
	
	if( e->size().height() < 776 )
		B->move( e->size().width() - 292-sidebar, e->size().height()-198 );
	else
		B->move( e->size().width() - 292-sidebar, 578 );

	if( e->size().height() < 837)
		El->move( e->size().width() - 292-sidebar, e->size().height()-403 );
	else
		El->move( e->size().width() - 292-sidebar, 434 );
	
//	--- Funktionen ---
	FL->move( e->size().width() - 292, 2 );
	FK->move( e->size().width() - 292, 50 );
	FE->move( e->size().width() - 292, 98 );
	}

void PCNCView::anfang_clicked()
	{
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	if( aM != -1 )
		{
		ncbefehlseditor->setCursorPosition( 0, 0 );
		ncbefehlseditor->setSelection ( 0, 0, 8, 0 );
		if( KMessageBox::No == KMessageBox::questionYesNo( 0, QString("Es ist schon eine Maschine ausgewhlt !\nDiese ersetzen ?"), "Fehler") )
			{
			ncbefehlseditor->selectAll ( FALSE );
			ncbefehlseditor->setCursorPosition( zeile, spalte );
			return;
			}
		}
		
	ma->exec();
	
	if( ! ma->MCHOSEN )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie mssen eine Maschine auswhlen, sonst kann keine Werkzeugdatei dem Auswahlknopf zugeordnet werden !"), "Fehler ");
		return;
		}
	
	line = "-Ausgewhlte Maschine:\n-";
	line += ma->name;
	line += "\nSPIEL\n X	"+QString::number( ma->spx )+"\n Y	"+QString::number( ma->spy )+"\n Z	"+QString::number( ma->spz )+"\n";
	line += "BESCHLEUNIGUNG\n [mm/min]	"+QString::number( ma->a )+"\n";
	
	ncbefehlseditor->setCursorPosition( 0, 0 );
	ncbefehlseditor->insert( line );
	ncbefehlseditor->setCursorPosition( zeile+8, spalte );
	aM = ma->aM;
	mfaktor = ma->mf;
	steuerprog = ma->sdat;
	werkzeug = ma->wdat;
	}

void PCNCView::ende_clicked()
	{
	ncbefehlseditor->insertParagraph ( "ENDE", -1 );
	}

void PCNCView::ende_einfuegen_clicked()
	{
	
	
	line="}SCHLEIFE\n anzahl	"+QString::number( S->anzahl->value() )+"\n";
	ncbefehlseditor->insert( line );
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	
	ncbefehlseditor->setFocus();
	m = 14; S->hide();
	}

void PCNCView::anfang_einfuegen_clicked()
	{
	line="SCHLEIFE{\n";
	ncbefehlseditor->insert( line );
	m = 14; S->hide();
	}

void PCNCView::maschiene()
	{
	if( ! mtool->isRunning() )
		{
		if( ! mtool->start( mtool->OwnGroup ) )
			{
			mtool->kill();
			KMessageBox::messageBox( 0, KMessageBox::Error, QString("<p>Konnte das Maschinentool nicht starten <br><center>Fehler : %1</center></p>").arg( mtool->exitStatus() ), "Fehler ");
			}
		return;
		}
	if( KMessageBox::Yes == KMessageBox::questionYesNo( 0, QString("Es ist ein Problem mit dem Maschinentool aufgetreden !\nSoll versucht werden das Maschinentool zu benden ?"), "Fehler") )
		mtool->kill();
	return;
	}

void PCNCView::werkzeuge()
	{
	if( ! wtool->isRunning() )
		{
		if( ! wtool->start( wtool->OwnGroup ) )
			{
			wtool->kill();
			KMessageBox::messageBox( 0, KMessageBox::Error, QString("<p>Konnte das Werkzeugtool nicht starten <br><center>Fehler : %1</center></p>").arg( wtool->exitStatus() ), "Fehler ");
			}
		return;
		}
	if( KMessageBox::Yes == KMessageBox::questionYesNo( 0, QString("Es ist ein Problem mit dem Werkzeugtool aufgetreden !\nSoll versucht werden das Werkzeugtool zu benden ?"), "Fehler") )
		wtool->kill();
	return;
	}

void PCNCView::bohrung_clicked()
	{
	if( aM != -1 )
		{
		Bo->posx->setMaxValue( ma->wx );
		Bo->posy->setMaxValue( ma->wy );
		Bo->dz->setMaxValue( ma->wz );
		Bo->vs->setMaxValue( ma->vs_eintauchen );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Bo->werkzeug->currentItem();
		Bo->werkzeug->clear();
		Bo->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_BOHRER == ( W_BOHRER & verwendung ) )
				Bo->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Bo->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Bo,  lastwidget_left);
	
	}

void PCNCView::bohrung_einfuegen_clicked()
	{
	int i, zeile, spalte;
	double dx, dy, dz;
	QString w;
	QFile f1( werkzeug );
	Q_INT16 verwendung;
	QCString name;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Bo->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr das Topfloch !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Bo->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Bo->werkzeug->currentItem() != 0 )
		{
		w = Bo->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			return;
		
		f1.close();
			
		if( W_BOHRER != ( W_BOHRER & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr das Bohren nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	dx = Bo->posx->value() - X;
	dy = Bo->posy->value() - Y;
	dz = Bo->dz->value();
	X = Bo->posx->value();
	Y = Bo->posy->value();
	
	line = ("------Bohrung------\n");
	if( dx != 0 || dy != 0 )
		line += "VORSCHUB\n [mm/min] :	" + QString::number( ma->v_max ) + "\nLINIE\n dX :	"+QString::number( dx, 'f', 2 )+"\n dY :	"+QString::number( dy, 'f', 2 )+"\n dZ :	0.00\n";

	line += "VORSCHUB\n [mm/min] :	" + QString::number( ma->vz ) + "\nLINIEZ\n dZ :	-1.00\nVORSCHUB\n [mm/min] :	" + QString::number( Bo->vs->value() ) + "\n";
	
	if( dz < 1.00 )
		line += "LINIEZ\n dZ :	" + QString::number( -dz, 'f', 2 ) +"\n";
	else
		{
		i = int( trunc( dz ) - 1 );
		if( i > 0 )
			line += "SCHLEIFE{\nLINIEZ\n dZ :	-1.00\nLINIEZ\n dZ :	0.05\nLINIEZ\n dZ :	-0.05\n}SCHLEIFE\n  anz:	" + QString::number( i ) +"\n";
		else
			line = "LINIEZ\n dZ :	-1.00\nLINIEZ\n dZ :	0.05\nLINIEZ\n dZ :	-0.05\n";
		
		if( dz > ( i + 1 ) )
			line += "LINIEZ\n dZ :	" + QString::number( -(dz-(i+1)), 'f', 2 ) +"\n";
		}
	
	line += "VORSCHUB\n [mm/min] :	" + QString::number( ma->vz ) +"\nLINIEZ\n dZ :	" + QString::number( dz+1.00, 'f', 2 ) +"\n";
	
	ncbefehlseditor->insert( line );
	Bo->hide(); n = 0;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::fase_clicked()
	{
	if( aM != -1 )
		{
		Bf->posx->setMaxValue( ma->wx );
		Bf->posy->setMaxValue( ma->wy );
		Bf->fase->setMaxValue( ma->wz );
		Bf->D->setMaxValue( ma->wy );
		Bf->vs->setMaxValue( ma->vs_eintauchen );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Bf->werkzeug->currentItem();
		Bf->werkzeug->clear();
		Bf->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_FASER == ( W_FASER & verwendung ) )
				Bf->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Bf->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Bf,  lastwidget_left);
	
	}

void PCNCView::fase_einfuegen_clicked()
	{
	int i;
	double dx, dy, dz, s;
	QString w;
	QFile f1( werkzeug );
	Q_INT16 verwendung;
	QCString name;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Bf->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr das Fasen !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Bf->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Bf->werkzeug->currentItem() != 0 )
		{
		w = Bf->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			{
			verwendung = 0;
			}
		f1.close();
			
		if( W_FASER != ( W_FASER & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr das Fasen nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	dx = Bf->posx->value() - X;
	dy = Bf->posy->value() - Y;
	dz = Bf->fase->value();
	s = Bf->D->value() / 2;
	X = Bf->posx->value();
	Y = Bf->posy->value();
	
	if( Bf->D->value() == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen den Durchmesser anzugeben !"), "Fehler ");
		return;
		}
	
	if( Bf->fase->value() == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen die Fase anzugeben !"), "Fehler ");
		return;
		}
	
	line = ("-----Bohrung fasen-----\n");
	if( dx != 0 || dy != 0 )
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\n  dX :	"+QString::number( dx, 'f', 2 )+"\n  dY :	"+QString::number( dy, 'f', 2 )+"\n  dZ :	"+QString::number( 0, 'f', 2 )+"\n";
	
	line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n  dZ :	"+QString::number( -1 - s, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Bf->vs->value() )+"\nLINIEZ\n  dZ :	"+QString::number( -dz, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n  dZ :	"+QString::number( dz +1 + s, 'f', 2 )+"\n";
	
	ncbefehlseditor->insert( line );
	Bf->hide(); n = 2;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::versenken_clicked()
	{
	if( aM != -1 )
		{
		Bv->posx->setMaxValue( ma->wx );
		Bv->posy->setMaxValue( ma->wy );
		Bv->dz->setMaxValue( ma->wz );
		Bv->vs->setMaxValue( ma->vs_eintauchen );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Bv->werkzeug->currentItem();
		Bv->werkzeug->clear();
		Bv->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_SENKER == ( W_SENKER & verwendung ) )
				Bv->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Bv->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Bv,  lastwidget_left);
	
	}

void PCNCView::versenken_einfuegen_clicked()
	{
	int i;
	double dx, dy;
	QString w;
	QFile f1( werkzeug );
	Q_INT16 verwendung;
	QCString name;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Bv->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr das Versenken !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Bv->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Bv->werkzeug->currentItem() != 0 )
		{
		w = Bv->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			{
			verwendung = 0;
			}
		f1.close();
			
		if( W_SENKER  != ( W_SENKER & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr das Versenken nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	dx = Bv->posx->value() - X;
	dy = Bv->posy->value() - Y;
	X = Bv->posx->value();
	Y = Bv->posy->value();
	
	line = ("----Bohrung versenken----\n");
	if( dx != 0 || dy != 0 )
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\n  dX :	"+QString::number( dx, 'f', 2 )+"\n  dY :	"+QString::number( dy, 'f', 2 )+"\n  dZ :	"+QString::number( 0, 'f', 2 )+"\n";
	
	line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vz ) + "\nLINIEZ\n  dZ :	-1.00\nVORSCHUB\n  [mm/min] :	"+QString::number( Bv->vs->value() ) + "\nLINIEZ\n  dZ :	"+QString::number( -Bv->dz->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vz ) +"\nLINIEZ\n  dZ :	"+QString::number( Bv->dz->value()+1, 'f', 2 ) +"\n";
	
	ncbefehlseditor->insert( line );
	Bv->hide(); n = 4;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::topfloch_clicked()
	{
	if( aM != -1 )
		{
		To->posx->setMaxValue( ma->wx );
		To->posy->setMaxValue( ma->wy );
		To->dz->setMaxValue( ma->wz );
		To->vs->setMaxValue( ma->v_max );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = To->werkzeug->currentItem();
		To->werkzeug->clear();
		To->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_TASCHE == ( W_TASCHE & verwendung ) )
				To->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		To->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( To,  lastwidget_left);
	
	}

void PCNCView::topfloch_einfuegen_clicked()
	{
	int i;
	unsigned long int  xn;
	double dx, dy, l2, D, d, h=0, R=0, r=0, laenge=0, precision=0;
	char kreisdev[9];
	QString w;
	QFile f1( werkzeug );
	Q_INT16 verwendung;
	QCString name;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( To->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr das Topfloch !"), "Fehler ");
		return;
		}
	
	if( i == -1 && To->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( To->werkzeug->currentItem() != 0 )
		{
		w = To->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			return;
		
		f1.close();
			
		if( W_TASCHE != ( W_TASCHE & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr das Bohren nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	if( f1.open( IO_ReadOnly ) ) 
		{
		f1.at( aW * 79 );
		str1 >> name >> verwendung >> d >> l2;
		}
	f1.close();
	
	for( i=0; i < 8; i++)
		{
		kreisdev[i] = '0';
		}
	kreisdev[i] = 0;
	
	if( ma->op & M_KUGELROLL )
		kreisdev[5] = '1';
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	dx = To->posx->value() - X;
	X = To->posx->value();
	dy = To->posy->value() - Y;
	Y = To->posy->value();
	D = To->D->value();
	
	if( To->dz->value() == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen die Tiefe anzugeben !"), "Fehler ");
		return;
		}
	
	if( D == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Durchmesser anzugeben !"), "Fehler ");
		return;
		}
	
	line = "--- Topfloch ---\n";
	
	if( dx != 0 || dy !=0 )
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\ndX	"+QString::number( dx, 'f', 2 )+"\ndY	"+QString::number( dy, 'f', 2 )+"\ndZ	0.00\n";
	
	line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n dZ	 -1.00\n";
	
	
	if( To->dz->value() > To->schlichten->value() )
		h = To->dz->value() - To->schlichten->value();
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Die Tiefe ist nicht ausreichend um zu Schlichten !"), "Fehler ");
		return;
		}
	
	if( d > D/3 )
		if( d > D/2 )
			if( d > D - 2 * To->schlichten->value() )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Suchen Sie einen Frser mit kleinerem Durchmesser heraus !"), "Fehler ");
				return;
				}
	// D > d > D/2 ---helix / Schnecke------------------
			else
				{
				r = ( ( D - d ) / 2 ) - To->schlichten->value();
				w = GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
				line += "LINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\n";
				
				if( h <= To->ds->value() )
					{
					xn = long( ceil( h * 100.00 ) );
					precision = ( int( c1.steps / ( xn * mfaktor ) ) + 1 ) / 100.00 ;   // in mm
					line += "SCHLEIFE{\nKREIS\n";
					line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], precision, 1, r, 0, 1, r, 0, &c1 );
					line += "LINIEZ\n dZ	"+QString::number( -0.01, 'f', 2 )+"\n}SCHLEIFE\nwiederhole	"+QString::number( xn - 1 )+"\n";
					}
				else
					{   // h > To->topfloch_zustellen->value()
					xn = long( ceil( h * 100 ) );   // loops Z - 0.01
					laenge = long( ceil( h / To->ds->value() ) * c1.steps );   // all steps
					precision = ceil( laenge / ( xn * mfaktor ) ) / 100 ;   // in mm
					line += "SCHLEIFE{\nKREIS\n kreisdev	00000000\n przision	"+QString::number( precision, 'f', 2 )+"\n x	"+QString::number( r, 'f', 2 )+"\n y	0.00\n radius	"+QString::number( r, 'f', 2 )+"\n schritte	"+QString::number( laenge )+"\n- dx=0.000  dy=0.000\n- n="+QString::number( xn )+"\n";
					line += "LINIEZ\n dZ	"+QString::number( -0.01, 'f', 2 )+"\n}SCHLEIFE\nwiederhole	"+QString::number( xn -1)+"\n";
					}
				
				line += "KREIS\n";
				line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
				line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\nLINIE\n dX	"+QString::number( To->schlichten->value(), 'f', 2 )+"\n dY	0.00\n dZ	"+QString::number( -(To->schlichten->value()), 'f', 2 )+"\nKREIS\n";
				r = ( D - d ) / 2;
				line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
				line += "VORSCHUB\n [mm/min] :	"+QString::number( ma->vz )+"\nLINIE\n dX	"+QString::number( -r, 'f', 2 )+"\n dY	0.00\n dZ	"+QString::number( To->dz->value()+1, 'f', 2 )+"\n";
				}
	
	// D durch 2 >= d ----------------------
		else
			{
			r = ( D - d ) / 2 - To->schlichten->value();
			w = GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
			h = To->dz->value() - To->schlichten->value();
			i = 0;
			while( h > To->ds->value() )
				{
				h -= To->ds->value();
				i ++;
				}   // h = rest
			
			if( i > 0 )
				{
				line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( - To->ds->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\nLINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
				line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
				line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIEX\n dX	"+QString::number( -r, 'f', 2 )+"\n}SCHLEIFE\n  wiederhole	"+QString::number( i - 1 )+"\n";
				}
			
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -h, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\nLINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIEX\n dX	"+QString::number( -r, 'f', 2 )+"\n";
			r = ( D - d ) / 2;
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -To->schlichten->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\nLINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 1, r, 0, 1, r, 0, &c1 );
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIE\n dX	"+QString::number( -r, 'f', 2 )+"\n dY	0.00\n dZ	"+QString::number( To->dz->value()+1, 'f', 2 )+"\n";
			}

	// D durch 3 >= d ----------------------
	else
		{
		p = fitsin( h, To->ds->value(), &h );
		dx = 0.00;
		R = ( ( D - d ) / 2.00 ) - To->schlichten->value();
		r = d / 2.00;
		if( p > 0 )
			{
			p = p - 1;
			if( p == 0 )
				line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -To->ds->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\n";
			else
				line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -To->ds->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\n";
			
			while( R > r )
				{
				R = R - r;
				dx = dx + r;
				line += "LINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
				line += GetCircle::getCircle( mfaktor,  dx, &kreisdev[0], 0, 1, dx, 0, 1, dx, 0, &c1 );
				}
			
			R = ( ( D - d ) / 2 ) - To->schlichten->value();
			line += "LINIEX\n dX	"+QString::number( R-dx, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  R, &kreisdev[0], 0, 1, R, 0, 1, R, 0, &c1 );
			
			if( p == 0 )
				line += "LINIEX\n dX	"+QString::number( -R, 'f', 2 )+"\n";
			else
				line += "LINIEX\n dX	"+QString::number( -R, 'f', 2 )+"\n}SCHLEIFE\n  wiederhole	"+QString::number( p )+"\n";
			}
		
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -h, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( To->vs->value(), 'f', 0 )+"\n";
		dx = 0.00;
		while( R > r )
			{
			R = R - r;
			dx = dx + r;
			line += "LINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  dx, &kreisdev[0], 0, 1, dx, 0, 1, dx, 0, &c1 );
			}
		
		R = ( D - d ) / 2 - To->schlichten->value();
		line += "LINIEX\n dX	"+QString::number( R-dx, 'f', 2 )+"\nKREIS\n";
		line += GetCircle::getCircle( mfaktor,  R, &kreisdev[0], 0, 1, R, 0, 1, R, 0, &c1 );
		line += "LINIEX\n dX	"+QString::number( -R, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -To->schlichten->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\n";
		
		// Schlichten ----------------------
		dx = 0.00;
		R = ( D - d ) / 2.00;
		while( R >= r )
			{
			R = R - r;
			dx = dx + r;
			line += "LINIEX\n dX	"+QString::number( r, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  dx, &kreisdev[0], 0, 1, dx, 0, 1, dx, 0, &c1 );
			}
		
		if( R > 0.00 )
			{
			R = ( D - d ) / 2.00;
			line += "LINIEX\n dX	"+QString::number( R-dx, 'f', 2 )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  R, &kreisdev[0], 0, 1, R, 0, 1, R, 0, &c1 );
			}
		
		R = ( D - d ) / 2.00;
		line += ("VORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIE\n dX	"+QString::number( -R, 'f', 2 )+"\n dY	0.00\n dZ	"+QString::number( To->dz->value()+1, 'f', 2 )+"\n");
		}
	
	ncbefehlseditor->insert( line );
	To->hide(); n = 6;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::zylinder_clicked()
	{
	
	if( aM != -1 )
		{
		Zy->posx->setMaxValue( ma->wx );
		Zy->posy->setMaxValue( ma->wy );
		Zy->dz->setMaxValue( ma->wz );
		Zy->vs->setMaxValue( ma->v_max );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Zy->werkzeug->currentItem();
		Zy->werkzeug->clear();
		Zy->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_TASCHE == ( W_TASCHE & verwendung ) )
				Zy->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Zy->werkzeug->setCurrentItem( n );
		}
	f1.close();

	lastwidget_left = showclicked( Zy,  lastwidget_left);
	
	}

void PCNCView::zylinder_einfuegen_clicked()
	{
	char kreisdev[9];
	int i;
	double xn, dx, dy, D, d, h=0.00, r=0.00;
	QString w;
	QFile f1( werkzeug );
	Q_INT16 verwendung;
	QCString name;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Zy->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr den Zylinder !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Zy->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Zy->werkzeug->currentItem() != 0 )
		{
		w = Zy->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			return;
		
		f1.close();
			
		if( W_TASCHE != ( W_TASCHE & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr den Zylinder nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	for( i=0; i < 8; i++)
		{
		kreisdev[i] = '0';
		}
	kreisdev[i] = 0;
	
	if( ( ma->op & M_KUGELROLL ) == M_KUGELROLL )
		kreisdev[5] = '1';
	
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		i = aW;
		f1.at( i * 79 );
		str1 >> name >> verwendung >> d;
		}
	f1.close();
	
	D = Zy->D->value();
	r = ( D + d ) / 2 + Zy->schlichten->value();
	dx = Zy->posx->value() - X + r;
	dy = Zy->posy->value() - Y;
	
	if( D == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Durchmesser anzugeben !"), "Fehler ");
		return;
		}
	
	line = ("--- Zylinder ---\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\ndX	"+QString::number( dx, 'f', 2 )+"\ndY	"+QString::number( dy, 'f', 2 )+"\ndZ	0.00\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n dZ	"+QString::number( -1.00, 'f', 2 )+"\n");

	h = Zy->dz->value() - Zy->schlichten->value();
	
	if( h < Zy->zustellen->value() )
		{
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -h, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Zy->vs->value() )+"\nKREIS\n";
		line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 4, r, 0, 4, r, 0, &c1 );
		}
	else
		{
		xn = -1;
		while( h >= Zy->zustellen->value() )
			{
			xn ++;
			h -= Zy->zustellen->value();
			}
		
		if( xn == 0 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -Zy->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Zy->vs->value() )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 4, r, 0, 4, r, 0, &c1 );
			}
		else
			{
			line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -Zy->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Zy->vs->value() )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 4, r, 0, 4, r, 0, &c1 );
			line += "}SCHLEIFE\nwiederhole	"+QString::number( xn, 'f', 0 )+"\n";
			}
		
		if( h > 0.00 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dZ	"+QString::number( -h, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Zy->vs->value() )+"\nKREIS\n";
			line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 4, r, 0, 4, r, 0, &c1 );
			}
		}
	
	r = ( D + d ) / 2;
	
	line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIE\n dX	"+QString::number( -Zy->schlichten->value(), 'f', 2 )+"\n dY	0.00\n dZ	"+QString::number( -Zy->schlichten->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\nKREIS\n";
	line += GetCircle::getCircle( mfaktor,  r, &kreisdev[0], 0, 4, r, 0, 4, r, 0, &c1 );
	line += "VORSCHUB\n [mm/min] :	"+QString::number( ma->vz )+"\nLINIEX\n dX	"+QString::number( Zy->schlichten->value(), 'f', 2 )+"\nLINIEZ\n dZ	"+QString::number( Zy->dz->value()+1, 'f', 2 )+"\n";
	
	
	ncbefehlseditor->insert( line );
	Zy->hide(); n = 8;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::quader_clicked()
	{
	if( aM != -1 )
		{
		Qu->x1->setMaxValue( ma->wx );
		Qu->y1->setMaxValue( ma->wy );
		Qu->x2->setMaxValue( ma->wx );
		Qu->y2->setMaxValue( ma->wy );
		Qu->dz->setMaxValue( ma->wz );
		Qu->vs->setMaxValue( ma->v_max );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Qu->werkzeug->currentItem();
		Qu->werkzeug->clear();
		Qu->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_TASCHE == ( W_TASCHE & verwendung ) )
				Qu->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Qu->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Qu,  lastwidget_left);
	
	}

void PCNCView::quader_einfuegen_clicked()
	{
	int zeile, spalte, i, p, zn, n;
	double dx, dy, dz, d=0.00, l2=0.00, l3=0.00;
	QString w;
	QFile f1( werkzeug );
	QCString name;
	Q_INT16 verwendung;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Qu->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr den Quader !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Qu->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Qu->werkzeug->currentItem() != 0 )
		{
		w = Qu->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			return;
		
		f1.close();
			
		if( W_TASCHE != ( W_TASCHE & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr den Quader nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	if( Qu->x1->value() > Qu->x2->value() )
		{
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "X1 ist grer als X2.\nDie Werte vertauschen ?", "Fehler", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			return;
		
		dx = Qu->x2->value();
		Qu->x2->setValue( Qu->x1->value() );
		Qu->x1->setValue( dx );
		}
	
	if( Qu->y1->value() > Qu->y2->value() )
		{
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Y1 ist grer als Y2.\nDie Werte vertauschen ?", "Fehler", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			return;
		
		dy = Qu->y2->value();
		Qu->y2->setValue( Qu->y1->value() );
		Qu->y1->setValue( dy );
		}
	
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		f1.at( aW * 79 );
		str1 >> name >> verwendung >> d >> l2 >> l3;
		f1.close();
		}
	
	if( l2 < Qu->dz->value() )
		if( Qu->schlichten->value() != 0 )
			{
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Frser kann in dieser Tiefe nicht mehr schlichten - L2 ist zu klein !"), "Fehler ");
			return;
			}
		else
			if( l3 < Qu->dz->value() )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Frser kann nicht so tief frsen !"), "Fehler ");
				return;
				}
			else
				if( KMessageBox::Yes == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Die Kante kann den Frser ber den Schneiden berhren - Abbrechen ?", "Werkzeug ", KStdGuiItem::yes(), KStdGuiItem::no() ) )
					return;
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	
	dx = Qu->x1->value() - X - d / 2.00 - Qu->schlichten->value();
	X = Qu->x1->value() - d / 2.00;
	dy = Qu->y1->value() - Y - d / 2.00 - Qu->schlichten->value();
	Y = Qu->y1->value() - d / 2.00;
	
	line = ("--- Quader ---\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	0.00\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n dZ	-1.00\n");
	
	dx = Qu->x2->value() - Qu->x1->value() + d + 2 * Qu->schlichten->value();
	dy = Qu->y2->value() - Qu->y1->value() + d + 2 * Qu->schlichten->value();
	dz = Qu->dz->value() - Qu->schlichten->value();
	n = 0;
	
	if( ma->op & M_KUGELROLL )
		{
		zn = div( int( dz * 100 ), int( Qu->zustellen->value() * 100 ) ).quot - 1;
		if( zn <= 0 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\n";
			
			if( dz <= Qu->zustellen->value() )
				line += "LINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\n";
			else
				line += "LINIEZ\n dz	"+QString::number( -Qu->zustellen->value(), 'f', 2 )+"\n";
			
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\n";
			}
		else   // zn > 0.00
			{
			line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -Qu->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\n}SCHLEIFE\n  wiederhole	"+QString::number( zn, 'f', 0 )+"\n";
			}
		
		// ---Den Rest beseitigen ---
		
		dx = Qu->x2->value() - Qu->x1->value() + d + 2 * Qu->schlichten->value();
		dy = Qu->y2->value() - Qu->y1->value() + d + 2 * Qu->schlichten->value();
		dz = Qu->dz->value() - Qu->schlichten->value() - ( zn + 1 ) * Qu->zustellen->value();
		n = 0;
		
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\n";
		}
	else   // keine Kugelrollspindeln
		{
		zn = div( int( dz * 100 ), int( Qu->zustellen->value() * 100 ) ).quot - 1;
		if( zn <= 0 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\n";
			
			if( dz <= Qu->zustellen->value() )
				line += "LINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\n";
			else
				line += "LINIEZ\n dz	"+QString::number( -Qu->zustellen->value(), 'f', 2 )+"\n";
			
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\n";
			}
		else   // zn > 0.00
			{
			line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -Qu->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\n}SCHLEIFE\n  wiederhole	"+QString::number( zn, 'f', 0 )+"\n";
			}
		
		// ---Den Rest beseitigen ---
		
		dx = Qu->x2->value() - Qu->x1->value() + d + 2 * Qu->schlichten->value();
		dy = Qu->y2->value() - Qu->y1->value() + d + 2 * Qu->schlichten->value();
		dz = Qu->dz->value() - Qu->schlichten->value() - ( zn + 1 ) * Qu->zustellen->value();
		n = 0;
		
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Qu->vs->value() )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\n";
		}
	
	// --- Schlichten ---
	if( Qu->schlichten->value() > 0 )
		{
		dx = Qu->x2->value() - Qu->x1->value() + d;
		dy = Qu->y2->value() - Qu->y1->value() + d;
		n = 0;
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\nLINIE\n dx	"+QString::number( Qu->schlichten->value(), 'f', 2 )+"\n dy	"+QString::number( Qu->schlichten->value(), 'f', 2 )+"\n dz	"+QString::number( -Qu->schlichten->value(), 'f', 2 )+"\n";
		
		if( ma->op & M_KUGELROLL )
			line += "LINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\n";
		
		else
			line += "LINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\n";
			
		line += "LINIEZ\n dz	"+QString::number( Qu->dz->value()+1, 'f', 2 )+"\n";
		}
	
	
	ncbefehlseditor->insert( line );
	
	Qu->hide(); n = 10;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::tasche_clicked()
	{
	
	if( aM != -1 )
		{
		Ta->x1->setMaxValue( ma->wx );
		Ta->y1->setMaxValue( ma->wy );
		Ta->x2->setMaxValue( ma->wx );
		Ta->y2->setMaxValue( ma->wy );
		Ta->dz->setMaxValue( ma->wz );
		Ta->vs->setMaxValue( ma->v_max );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Ta->werkzeug->currentItem();
		Ta->werkzeug->clear();
		Ta->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_TASCHE == ( W_TASCHE & verwendung ) )
				Ta->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Ta->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Ta,  lastwidget_left);
	
	}

void PCNCView::tasche_einfuegen_clicked()
	{
	unsigned int p;
	int zeile, spalte, zn, i, n;
	double dx=0.00, dy=0.00, dz=0.00, d=0.00, l2=0.00, l3=0.00;
	QString w;
	QFile f1( werkzeug );
	QCString name;
	Q_INT16 verwendung;
	QDataStream str1( &f1 );
	
	i = aW;
	
	if( Ta->werkzeug->count() == 1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben leider kein passendes Werkzeug fr die Tasche !"), "Fehler ");
		return;
		}
	
	if( i == -1 && Ta->werkzeug->currentItem() == 0 && ! defAW( &i ) )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Es werden Werkzeugdaten bentigt und das aktuelle Werkzeug ist unbekannt !"), "Fehler ");
		return;
		}
	
	if( Ta->werkzeug->currentItem() != 0 )
		{
		w = Ta->werkzeug->currentText();
		w.remove( 0, 1 );
		p = w.find( ')' );
		w.remove( p, w.length() - p );
		OK = TRUE;
		i = StrToNum::StringToShort( w, &OK, 0 );
		}
	else
		{
		if ( f1.open( IO_ReadOnly ) )
			{
			f1.at( i * 79 );
			str1 >> name >> verwendung;
			}
		else
			return;
		
		f1.close();
			
		if( W_TASCHE != ( W_TASCHE & verwendung ) )
			{
			if( FIRST )
				firstTool( i );
			
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Das aktuelle Werkzeug ist fr die Tasche nicht geeignet !"), "Fehler ");
			return;
			}
		}
	
	if( FIRST )
		firstTool( i );
	else
		if ( aW != i )
			werkzeugwechsel( i );
	
	if( Ta->x1->value() > Ta->x2->value() )
		{
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "X1 ist grer als X2.\nDie Werte vertauschen ?", "Fehler", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			return;
		
		dx = Ta->x2->value();
		Ta->x2->setValue( Ta->x1->value() );
		Ta->x1->setValue( dx );
		}
	
	if( Ta->y1->value() > Ta->y2->value() )
		{
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Y1 ist grer als Y2.\nDie Werte vertauschen ?", "Fehler", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			return;
		
		dy = Ta->y2->value();
		Ta->y2->setValue( Ta->y1->value() );
		Ta->y1->setValue( dy );
		}
	
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		i = aW;
		f1.at( i * 79 );
		str1 >> name >> verwendung >> d >> l2 >> l3;
		f1.close();
		}
	
	if( l2 < Ta->dz->value() )
		if( Ta->schlichten->value() != 0 )
			{
			KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Frser kann in dieser Tiefe nicht mehr schlichten - L2 ist zu klein !"), "Fehler ");
			return;
			}
		else
			if( l3 < Ta->dz->value() )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Frser kann nicht so tief frsen !"), "Fehler ");
				return;
				}
			else
				if( KMessageBox::Yes == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Die Kante kann den Frser ber den Schneiden berhren - Abbrechen ?", "Werkzeug ", KStdGuiItem::yes(), KStdGuiItem::no() ) )
					return;
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	
	dx = Ta->x1->value() + Ta->schlichten->value() + d / 2.00 - X;
	X = Ta->x1->value() + d / 2.00;
	dy = Ta->y1->value() + Ta->schlichten->value() + d / 2.00 - Y;
	Y = Ta->y1->value() + d / 2.00;
	line = "--- Tasche ---\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->v_max )+"\nLINIE\n dx	"+QString::number( dx, 'f', 2 )+"\n dy	"+QString::number( dy, 'f', 2 )+"\n dz	0.00\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vz )+"\nLINIEZ\n dz	-1.00\n";

	dx = Ta->x2->value() - Ta->x1->value() - 2.00 * Ta->schlichten->value() - d;
	dy = Ta->y2->value() - Ta->y1->value() - 2.00 * Ta->schlichten->value() - d;
	dz = Ta->dz->value() - Ta->schlichten->value();
	n = 0;
	if( dx < d || dy < d )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Die Tasche kann mit diesem Frser nicht gefrst werden\nSuchen Sie einen kleineren Frser heraus !"), "Fehler ");
		return;
		}
	
	if( ma->op & M_KUGELROLL )
		{
		zn = div( int( dz * 100 ), int( Ta->zustellen->value() * 100 ) ).quot - 1;
		if( zn <= 0 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\n";
			
			if( dz <= Ta->zustellen->value() )
				line += "LINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\n";
			else
				line += "LINIEZ\n dz	"+QString::number( -Ta->zustellen->value(), 'f', 2 )+"\n";
			
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
			
			while( dx >= d || dy >= d )
				{
				line += "LINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
			
			line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n";
			}
		else   // zn > 0.00
			{
			line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -Ta->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
			
			while( dx >= d || dy >= d )
				{
				line += "LINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
			
			line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n}SCHLEIFE\n  wiederhole	"+QString::number( zn, 'f', 0 )+"\n";
			}
		
		// ---Den Rest beseitigen ---
		
		dx = Ta->x2->value() - Ta->x1->value() - 2.00 * Ta->schlichten->value() - d;
		dy = Ta->y2->value() - Ta->y1->value() - 2.00 * Ta->schlichten->value() - d;
		dz = Ta->dz->value() - Ta->schlichten->value() - Ta->zustellen->value() * ( zn + 1 );
		n = 0;
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
		
		while( dx >= d || dy >= d )
			{
			line += "LINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
			dx -= d;
			dy -= d;
			n ++;
			}
		
		line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n";
		}
	else   // Keine Kugelrollspindel
		{
		zn = div( int( dz * 100 ), int( Ta->zustellen->value() * 100 ) ).quot - 1;
		if( zn <= 0.00 )
			{
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\n";
			
			if( dz <= Ta->zustellen->value() )
				line += "LINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\n";
			else
				line += "LINIEZ\n dz	"+QString::number( -Ta->zustellen->value(), 'f', 2 )+"\n";
			
			line += "VORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
			
			while( dx >= d || dy >= d )
				{
				line += "LINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
			
			line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n";
			}
		else   // zn > 0.00
			{
			line += "SCHLEIFE{\nVORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -Ta->zustellen->value(), 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
			
			while( dx >= d || dy >= d )
				{
				line += "LINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
			
			line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n}SCHLEIFE\n  wiederhole	"+QString::number( zn, 'f', 0 )+"\n";
			}
		
		// ---Den Rest beseitigen ---
		
		dx = Ta->x2->value() - Ta->x1->value() - 2.00 * Ta->schlichten->value() - d;
		dy = Ta->y2->value() - Ta->y1->value() - 2.00 * Ta->schlichten->value() - d;
		dz = Ta->dz->value() - Ta->schlichten->value() - Ta->zustellen->value() * ( zn + 1 );
		n = 0;
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_eintauchen )+"\nLINIEZ\n dz	"+QString::number( -dz, 'f', 2 )+"\nVORSCHUB\n  [mm/min] :	"+QString::number( Ta->vs->value() )+"\n";
		while( dx >= d || dy >= d )
			{
			line += "LINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
			dx -= d;
			dy -= d;
			n ++;
			}
		
		line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	0.00\n";
		}
	
	// --- Schlichten ---
	if( Ta->schlichten->value() > 0 )
		{
		dx = Ta->x2->value() - Ta->x1->value() - d;
		dy = Ta->y2->value() - Ta->y1->value() - d;
		n = 0;
		line += "VORSCHUB\n  [mm/min] :	"+QString::number( ma->vs_schlichten )+"\nLINIE\n dx	"+QString::number( -Ta->schlichten->value(), 'f', 2 )+"\n dy	"+QString::number( -Ta->schlichten->value(), 'f', 2 )+"\n dz	"+QString::number( -Ta->schlichten->value(), 'f', 2 )+"\n";
		
		if( ma->op & M_KUGELROLL )
			while( dx >= d || dy >= d )
				{
				line += "LINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
		else
			while( dx >= d || dy >= d )
				{
				line += "LINIEX\n dx	"+QString::number( dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( dy, 'f', 2 )+"\nLINIEX\n dx	"+QString::number( -dx, 'f', 2 )+"\nLINIEY\n dy	"+QString::number( -dy, 'f', 2 )+"\nLINIE\n dx	"+QString::number( d/2, 'f', 2 )+"\n dy	"+QString::number( d/2, 'f', 2 )+"\n dz	0.00\n";
				dx -= d;
				dy -= d;
				n ++;
				}
			
		line += "LINIE\n dx	"+QString::number( -(n*d/2), 'f', 2 )+"\n dy	"+QString::number( -(n*d/2), 'f', 2 )+"\n dz	"+QString::number( Ta->dz->value()+1, 'f', 2 )+"\n";
		}
	
	Ta->hide(); n = 12; 
	ncbefehlseditor->insert( line );
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::text_clicked()
	{
	if( aM != -1 )
		{
		Tx->sh->setMaxValue( ma->s_max );
		Tx->za->setMaxValue( ma->s_max );
		Tx->tiefe->setMaxValue( ma->wz );
		}
	else
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt.\nWhlen Sie mit dem Knopf ""Anfang"" eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Q_INT16 verwendung = 0;
		int n = Tx->werkzeug->currentItem();
		Tx->werkzeug->clear();
		Tx->werkzeug->insertItem( "aktuelles Werkzeug" );
		while ( ! str1.atEnd()  )
			{
			str1 >> name >> verwendung;
			if( W_TASCHE == ( W_TASCHE & verwendung ) )
				Tx->werkzeug->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		Tx->werkzeug->setCurrentItem( n );
		}
	f1.close();
	
	lastwidget_left = showclicked( Tx,  lastwidget_left);

	}

void PCNCView::text_einfuegen_clicked()
	{
	n = 14; Tx->hide();
	KMessageBox::messageBox ( 0, KMessageBox::Information, QString("In Development !\nDieses Men wird erst im Zusammenhang mit ''Konturen aus Bildern importieren'' verfgbar sein !"), "Fehler ");
	}

void PCNCView::z_auswahl_clicked()
	{
	zeichendat = KFileDialog::getOpenFileName( QDir::homeDirPath(), "*.ttf *.TTF| TrueType-Schrift (.TTF)\n  *.*| Alle", this, "Maschinen-Datei ffnen");
	Tx->zeichensatz->setText( zeichendat );
	}

void PCNCView::kontur_button_clicked()
	{
	Konturdialog *konturfenster = new Konturdialog(this);
	konturfenster->exec();
	if( konturfenster->exitcode == 0 )
		{
		konturfenster->kontur_file.remove( 0, konturfenster->kontur_file.findRev("/")+1 );
		ncbefehlseditor->insert( "-Kontur "+konturfenster->kontur_file+"\n" );
		ncbefehlseditor->insert( konturfenster->editor->text() );
		int i, p;
		ncbefehlseditor->getCursorPosition ( &p, &i );
		if( i != 0 )
			ncbefehlseditor->insert( QString("\n") );
		ncbefehlseditor->insert( "-Konturende "+konturfenster->kontur_file+"\n" );
		}
	}

void PCNCView::read( )
	{
	fileNew();
	nc_datei = KFileDialog::getOpenFileName( dir+QString( "/Werkstcke" ), "*.nc| PCNC-Datei (.nc)\n  *.*| Alle", this, "Maschinen-Datei ffnen");
	FIRST = FALSE;
	
	QFile f1( nc_datei );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QTextStream str1( &f1 );
		ncbefehlseditor->setText(str1.read());
		}
	f1.close();
	
	QString st = ncbefehlseditor->text(0);
	
	if( st.at(1) == 'A' && st.at(21) == ':' )
		{
		st = ncbefehlseditor->text(1);
		st.remove( 0, 1 );
		st = st.stripWhiteSpace();
		
		QFile f2( "maschinen.dat" );
		if ( f2.open( IO_ReadOnly ) ) 
			{
			QDataStream str2( &f2 );
			char *name;
			int i = 0;
			
			while( ! str2.atEnd() )
				{
				str2 >> name;
				
				if( st == name )
					{
					str2 >> ma->spx >> ma->spy >> ma->spz >> ma->vx >> ma->vy >> ma->vz >> ma->wx >> ma->wy >> ma->wz >> ma->op >> ma->mf >> ma->sdat >> ma->wdat;
					Sp->x->setValue( ma->spx );
					Sp->y->setValue( ma->spy );
					Sp->z->setValue( ma->spz );
					Vrs->vs_max_x->setValue( ma->vx );
					Vrs->vs_max_y->setValue( ma->vy );
					Vrs->vs_max_z->setValue( ma->vz );
					Vrs->vs_schlichten->setValue( ma->vs_schlichten );
					Vrs->vs_eintauchen->setValue( ma->vs_eintauchen );
					mfaktor = ma->mf;
					steuerprog = ma->sdat;
					werkzeug = ma->wdat;
					aM = i;
					break;
					}
				i++;
				f2.at( i * 128 );
				}
			}
		f2.close();
		}
	}

void PCNCView::write()
	{
    QFile f1( nc_datei );
    if ( f1.open( IO_WriteOnly | IO_Truncate ) ) 
		{
		QTextStream str1( &f1 );
		str1 << ncbefehlseditor->text();
		ncbefehlseditor->setModified(FALSE);
		f1.close();
		}
	}

void PCNCView::writeto()
	{
	nc_datei = KFileDialog::getSaveFileName ( QDir::homeDirPath()+QString("/PCNC/Werkstcke/"), "*.nc| PCNC-Datei (.nc)\n  *.*| Alle", this, "sichern_als" );
	QFile f1( nc_datei );
	if ( f1.open( IO_WriteOnly | IO_Truncate ) ) 
		{
		QTextStream str1( &f1 );
		str1 << ncbefehlseditor->text();
		ncbefehlseditor->setModified(FALSE);
		f1.close();
		}
	}


void PCNCView::fileNew()
	{
	if( ncbefehlseditor->isModified() )
		if( KMessageBox::Yes == KMessageBox::questionYesNo( 0, QString("Die Datei ""%1"" wurde verndert !\nSpeichern ?").arg(nc_datei), "Achtung") )
			if( nc_datei == "Werkstcke/default.nc" )
				writeto();
			else
				write();
	
	ncbefehlseditor->clear();
	ncbefehlseditor->setModified(FALSE);
	X = 0;
	Y = 0;
	Z = 0;
	aW = -1;
	aM = -1;
	FIRST = TRUE;
	nc_datei = "Werkstcke/default.nc";
	}

void PCNCView::make_nc()
	{
	bool SCHLEIFE=FALSE;
	int i;
	QFile f1( "nc.tmp" );
	unsigned int sindex=0;
	unsigned short int us1;
	long int l1, l2, l3;
	unsigned long int ul1, ul2, ul3, ul4, ul5, schleife[255];
	
	if ( f1.open( IO_ReadWrite | IO_Truncate ) ) 
		{
		QDataStream str1( &f1 );
		str1.setByteOrder ( QDataStream::LittleEndian );
		i = 0;
		s = "etwas";
		while ( ! s.isEmpty() )
			{
			s = ncbefehlseditor->text(i);
			i++;
			s = s.stripWhiteSpace();
			if( s.at(0) == '-' );
			else
			if( ! s.compare("ENDE") )
				{
				c = 0;
				str1 << c;
				}
			else
			if( ! s.compare( "LINIE" ) )
				{
				c = 1;
				str1 << c;
				schleif ++;
				c = 0;
				l1 = getlong( i ); i++; if( !OK ) break;
				l2 = getlong( i ); i++; if( !OK ) break;
				l3 = getlong( i ); i++; if( !OK ) break;
				if( l1 < 0 ){ c = c | bit0; l1 = -l1;}
				if( l2 < 0 ){ c = c | bit1; l2 = -l2;}
				if( l3 < 0 ){ c = c | bit2; l3 = -l3;}
				str1 << c << l1 << l2 << l3;
				schleif ++;
				}
			else
			if( ! s.compare ( "LINIEX" ) )
				{
				c = 2;
				str1 << c;
				schleif ++;
				c = 0;
				l1 = getlong( i ); i++; if( !OK ) break;
				if( l1 < 0 ){ c = c | bit0; l1 = -l1;}
				str1 << c << l1;
				schleif ++;
				}
			else
			if( ! s.compare ( "LINIEY" ) )
				{
				c = 3;
				str1 << c;
				schleif ++;
				c = 0;
				l1 = getlong( i ); i++; if( !OK ) break;
				if( l1 < 0 ){ c = c | bit1; l1 = -l1;}
				str1 << c << l1;
				schleif ++;
				}
			else
			if( ! s.compare ( "LINIEZ" ) )
				{
				c = 4;
				str1 << c;
				schleif ++;
				c = 0;
				l1 = getlong( i ); i++; if( !OK ) break;
				if( l1 < 0 ){ c = c | bit2; l1 = -l1;}
				str1 << c << l1;
				schleif ++;
				}
			else
			if( ! s.compare ( "KREIS" ) )
				{
				c = 5;
				str1 << c;
				schleif ++;                                       // KREIS
				c = getbits( i ); i++; if( !OK ) break;
				us1 = getushort( i ); i++; if( !OK ) break;
				ul1 = getulong( i ); i++; if( !OK ) break;
				ul2 = getulong( i ); i++; if( !OK ) break;
				ul3 = getulong( i ); i++; if( !OK ) break;
				
				OK=TRUE;
				s = ncbefehlseditor->text( i ); i++;
				p = s.find( '	' ) + 1;
				s.remove( 0, p );
				s = s.stripWhiteSpace();
				if( s.isEmpty() )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert anzugeben !").arg( s ), "Fehler ");
					ncbefehlseditor->insertParagraph( "Wert	0", i );
					ncbefehlseditor->setCursorPosition ( i, ncbefehlseditor->paragraphLength( i ) - 1 );
					OK=FALSE;
					break;
					}
				ul4 = StrToNum::StringToULong( s, &OK, 0 );
				if( !OK )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Die Zahl '%1' konnte nicht entschlsselt werden !").arg( s ), "Fehler ");
					ncbefehlseditor->setCursorPosition ( i, p );
					break;
					}
				schleif = schleif + 4;
				
				str1 << c << us1 << ul1 << ul2 << ul3 << ul4;
				}
			else
			if( ! s.compare ( "ELLIPSE" ) )
				{
				c = 6;
				str1 << c;                                       // ELLIPSE
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				us1 = getushort( i ); i++; if( !OK ) break;
				ul1 = getulong( i ); i++; if( !OK ) break;
				ul2 = getulong( i ); i++; if( !OK ) break;
				ul3 = getulong( i ); i++; if( !OK ) break;
				ul4 = getulong( i ); i++; if( !OK ) break;
								
				OK=TRUE;
				s = ncbefehlseditor->text( i ); i++;
				p = s.find( '	' ) + 1;
				s.remove( 0, p );
				s = s.stripWhiteSpace();
				if( s.isEmpty() )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert anzugeben !").arg( s ), "Fehler ");
					ncbefehlseditor->insertParagraph( "Wert	0", i );
					ncbefehlseditor->setCursorPosition ( i, ncbefehlseditor->paragraphLength( i ) - 1 );
					OK=FALSE;
					break;
					}
				ul5 = StrToNum::StringToULong( s, &OK, 0 );
				if( !OK )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Die Zahl '%1' konnte nicht entschlsselt werden !").arg( s ), "Fehler ");
					ncbefehlseditor->setCursorPosition ( i, p );
					break;
					}
				
				schleif = schleif + 4;
				str1 << c << us1 << ul1 << ul2 << ul3 << ul4 << ul5;
				}
			else
			if( ! s.compare ( "WARTE" ) )
				{
				c = 7;
				str1 << c;
				schleif ++;
				}
			else
			if( ! s.compare ( "STEUERUNG" ) )
				{
				c = 8;
				str1 << c;
				schleif ++;
				}
			else
			if( ! s.compare ( "SPIEL" ) )
				{
				c = 9;
				str1 << c;
				schleif ++;
				c = getuchar( i ); i++; if( !OK ) break;
				str1 << c;
				c = getuchar( i ); i++; if( !OK ) break;
				str1 << c;
				c = getuchar( i ); i++; if( !OK ) break;
				str1 << c;
				}
			else
			if( ! s.compare ( "KREISFUNKTION" ) )
				{
				c = 10;
				str1 << c;
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				us1 = getushort( i ); i++; if( !OK ) break;
				ul1 = getulong( i ); i++; if( !OK ) break;
				ul2 = getulong( i ); i++; if( !OK ) break;
				ul3 = getulong( i ); i++; if( !OK ) break;
				ul4 = getulong( i ); i++; if( !OK ) break;
				str1 << c << us1 << ul1 << ul2 << ul3 << ul4;
				}
			else
			if( ! s.compare ( "ELLIPSENFUNKTION" ) )
				{
				c = 11;
				str1 << c;
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				us1 = getushort( i ); i++; if( !OK ) break;
				ul1 = getulong( i ); i++; if( !OK ) break;
				ul2 = getulong( i ); i++; if( !OK ) break;
				ul3 = getulong( i ); i++; if( !OK ) break;
				ul4 = getulong( i ); i++; if( !OK ) break;
				ul5 = getulong( i ); i++; if( !OK ) break;
				str1 << c << us1 << ul1 << ul2 << ul3 << ul4 << ul5;
				}
			else
			if( ! s.compare ( "SCHLEIFE{" ) )
				{
				if( !SCHLEIFE )
					{
					schleif = 0;
					sindex = 0;
					}
				else
					{
					if( sindex == 255 ) KMessageBox::messageBox ( 0, KMessageBox::Information, QString(" Die Anzahl von 256 geschachtelten Schleifen ist erreicht.\n schreiben Sie Ihr Programm um und schicken Sie mir eine bse E-Mail !"), "Fehler ");
					schleife[sindex] = schleif;
					sindex++;
					schleif = 0;
					}
				SCHLEIFE=TRUE;
				}
			else
			if( ! s.compare ( "}SCHLEIFE" ) )
				{
				if( SCHLEIFE )
					{
					OK=TRUE;
					c = 12;
					str1 << c;
					ul1 = getulong( i, 0 ); i++; if( !OK ) break;
					schleif = schleif - 4;
					ul1 = ul1 / mfaktor;
					str1 << ul1 << schleif;
					schleif = schleif + 9;
					if( sindex > 0 )
						{
						sindex--;
						schleif = schleif + schleife[sindex];
						}
					else
						SCHLEIFE=FALSE;
					}
				else
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Schleifenanfang 'SCHLEIFE{' fehlt !"), "Fehler ");
					ncbefehlseditor->setCursorPosition ( i-1, 0 );
					break;
					}
				}
			else
			if( ! s.compare ( "DELTAKREIS" ) )
				{
				c = 13;
				str1 << c;
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				str1 << c;
				}
			else
			if( ! s.compare ( "DELTAELLIPS" ) )
				{
				c = 14;
				str1 << c;
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				str1 << c;
				}
			else
			if( ! s.compare ( "VORSCHUB" ) )
				{
				c = 15;
				str1 << c;
				schleif ++;
				ul1 = getulong( i, 0 ); i++; if( !OK ) break;
				ul1 = ul1 / 2;
				ul1 = (min_timesteps * ma->v_max ) / ul1;
				str1 << ul1;
				}
			else
			if( ! s.compare ( "DELTALINEAR" ) )
				{
				c = 16;
				str1 << c;
				schleif ++;
				c = getbits( i ); i++; if( !OK ) break;
				str1 << c;
				ul1 = getulong( i ); i++; if( !OK ) break;
				str1 << ul1;
				}
			else
			if( ! s.compare ( "WERKZEUGWECHSEL" ) )
				{
				c = 17;
				str1 << c;
				schleif ++;
				OK=TRUE;
				s = ncbefehlseditor->text( i ); i++;
				p = s.find( '	' ) + 1;
				s.remove( 0, p );
				s = s.stripWhiteSpace();
				if( s.isEmpty() )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert anzugeben !").arg( s ), "Fehler ");
					ncbefehlseditor->insertParagraph( "Wert	0", i );
					ncbefehlseditor->setCursorPosition ( i, ncbefehlseditor->paragraphLength( i ) - 1 );
					OK=FALSE;
					break;
					}
				ul1 = StrToNum::StringToULong( s, &OK, 0 );
				if( !OK )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Die Zahl '%1' konnte nicht entschlsselt werden !").arg( s ), "Fehler ");
					ncbefehlseditor->setCursorPosition ( i, p );
					break;
					}
				
				str1 << ul1;
				}
			else
			if( ! s.compare ( "MEMO" ) )
				{
				c = 18;
				str1 << c;
				schleif++;
				us1 = getushort( i, 0 ); i++; if( !OK ) break;
				str1 << us1;
				schleif += us1;
				s = ncbefehlseditor->text(i);
				i++;
				uint m=0, l=s.length();
				for( int n=0; n<us1; n++ )
					{
					QChar qc;
					if( m < l )
						{
						c = qc = s.at( m );
						str1 << c;
						m++;
						}
					else
						{
						m = 0;
						n--;
						s = ncbefehlseditor->text(i);
						i++;
						l = s.length();
						}
					}
				}
			else
			if( ! s.compare ( "BESCHLEUNIGUNG" ) )
				{
				c = 19;
				str1 << c;
				schleif++;
				us1 = getushort( i, 0 ); i++; if( !OK ) break;
				str1 << us1;
				}
			else
				{
				if( ! s.isEmpty() )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Befehl '%1' konnte nicht entschlsselt werden").arg( s ), "Fehler ");
					ncbefehlseditor->setCursorPosition ( i-1, 0 );
					break;
					}
				else
					{
					if( ncbefehlseditor->paragraphs() > i | i==1 )
						{
						if( KMessageBox::messageBox ( 0, KMessageBox::WarningYesNo, QString("<b>Eine leere Zeile gefunden !</b><br>Wenn Sie eine Trennungszeile wollen,<br>sollte diese mit '-' beginnen<br><br>Zeile Lschen ?"), "Fehler ") != 4)
							{
							ncbefehlseditor->removeParagraph( i-1);
							break;
							}
						else
							break;
						}
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("<p>Die Dekodierung war erfolgreich.<br><br><b></b><br>Die Steuerung wird jetzt gestartet.</p>"), "Info");
					l = 0;
					str1 << l;
					
					QFile f2( "s.tmp" );
					f2.open( IO_ReadWrite | IO_Truncate );
					QDataStream str2( &f2 );
					f1.at( 0 );
					Q_INT8 b;
					while( ! f1.atEnd() )
						{
						str1 >> b;
						str2 << b;
						}
						
					f1.close();
					f2.close();
					
					steur->clearArguments();
					if( 0 == getuid() )
						*steur << "konsole" << " --notabbar --nomenubar --noclose -e " << steuerprog;
					else
						*steur << "kdesu -c 'konsole  --notabbar --nomenubar --noclose -e '" << steuerprog;
					
					if( ! steur->isRunning() )
						{
						if( ! steur->start( steur->OwnGroup ) )
							{
							steur->kill();
							KMessageBox::messageBox ( 0, KMessageBox::Error, QString("<p>Konnte die Steuerung nicht starten <br><center>Fehler : %1</center></p>").arg( steur->exitStatus() ), "Fehler ");
							return;
							}
						return;
						}
					if( KMessageBox::Yes == KMessageBox::questionYesNo( 0, QString("Es ist ein Problem mit der Steuerung aufgetreden !\nSoll versucht werden die Steuerung zu benden ?"), "Fehler") )
						steur->kill();
					return;
					}
				}
			}
		}
	}

void PCNCView::linie_clicked()
	{
	if( aM != -1 )
		{
		Li->dx->setMaxValue( ma->wx );
		Li->dx->setMinValue( -ma->wx );
		Li->dy->setMaxValue( ma->wy );
		Li->dy->setMinValue( -ma->wy );
		Li->dz->setMaxValue( ma->wz );
		Li->dz->setMinValue( -ma->wz );
		Li->px->setMaxValue( ma->wx );
		Li->px->setMinValue( -ma->wx );
		Li->py->setMaxValue( ma->wy );
		Li->py->setMinValue( -ma->wy );
		Li->pz->setMaxValue( ma->wz );
		Li->pz->setMinValue( -ma->wz );
		Li->laenge->setMaxValue( ma->s_max );
		}
	
	lastwidget_right = showclicked( Li,  lastwidget_right);
	
	}

void PCNCView::linie_einfuegen_clicked()
	{
	double dx=0, dy=0, dz=0;
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	switch ( Li->basis->currentPageIndex() )
		{
		case 0:	// Lngenangaben
			dx = Li->dx->value();
			dy = Li->dy->value();
			dz = Li->dz->value();
			break;
		case 1:	// zum nchsten Punkt
			dx = Li->px->value() - X;
			dy = Li->py->value() - Y;
			dz = Li->pz->value() - Z;
			break;
		case 2:	// polar
			switch ( Li->achse->currentItem() )
				{
				case 0: 		// X-Achse
					switch ( Li->wert->currentItem() )
						{
						case 0:
							dx = fabs( Li->laenge->value() * cos( PI * Li->winkel1->value() / 180 ) * cos( PI * Li->winkel2->value() / 180 ) );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dy = - dx * tan( PI * Li->winkel1->value() / 180 );
							else
								dy = dx * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dz = - dx * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dx * tan( PI * Li->winkel2->value() / 180 );
							
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dx = -dx;
							
							break;
						case 1:
							dx = Li->laenge->value();
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dy = - dx * tan( PI * Li->winkel1->value() / 180 );
							else
								dy = dx * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dz = - dx * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dx * tan( PI * Li->winkel2->value() / 180 );
								
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dx = -dx;
								
							break;
						case 2:
							if( Li->winkel1->value() == 0 | Li->winkel1->value() == 180 | Li->winkel1->value() == -180 )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 1), kann es keinen Y-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel1->value() >= 0 )
								dy = Li->laenge->value();
							else
								dy = - Li->laenge->value();
							
							dx = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel1->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel2->value() ) > 90 ) ) )
								dz = - dx * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dx * tan( PI * Li->winkel2->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dx = - dx;
							
							break;
						case 3:
							if( ( Li->winkel2->value() == 0 | Li->winkel2->value() == 180 | Li->winkel2->value() == -180 ) )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 2), kann es keinen Z-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel2->value() >= 0 )
								dz = Li->laenge->value();
							else
								dz = - Li->laenge->value();
							
							dx = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel2->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel1->value() ) > 90 ) ) )
								dy = - dx * tan( PI * Li->winkel1->value() / 180 );
							else
								dy = dx * tan( PI * Li->winkel1->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dx = - dx;
							
							break;
						}
					break;
				
				case 1:		// Y-Achse
					switch ( Li->wert->currentItem() )
						{
						case 0:
							dy = fabs( Li->laenge->value() * cos( PI * Li->winkel1->value() / 180 ) * cos( PI * Li->winkel2->value() / 180 ) );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dx = - dy * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dy * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dz = - dy * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dy * tan( PI * Li->winkel2->value() / 180 );
							
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dy = -dy;
							
							break;
						
						case 1:
							if( Li->winkel1->value() == 0 | Li->winkel1->value() == 180 | Li->winkel1->value() == -180 )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 1), kann es keinen X-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel1->value() > 0 )
								dx = Li->laenge->value();
							else
								dx = - Li->laenge->value();
							
							dy = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel1->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel2->value() ) > 90 ) ) )
								dz = - dy * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dy * tan( PI * Li->winkel2->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dy = - dy;
							
							break;
						
						case 2:
							dy = Li->laenge->value();
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dx = - dy * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dy * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dz = - dy * tan( PI * Li->winkel2->value() / 180 );
							else
								dz = dy * tan( PI * Li->winkel2->value() / 180 );
								
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dy = -dy;
								
							break;
						
						case 3:
							if( ( Li->winkel2->value() == 0 | Li->winkel2->value() == 180 | Li->winkel2->value() == -180 ) )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 2), kann es keinen Z-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel2->value() > 0 )
								dz = Li->laenge->value();
							else
								dz = - Li->laenge->value();
							
							dy = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel2->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel1->value() ) > 90 ) ) )
								dx = - dy * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dy * tan( PI * Li->winkel1->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dy= - dy;
							
							break;
						}
					break;
					
				case 2:		// Z-Achse
					switch ( Li->wert->currentItem() )
						{
						case 0:
							dz = fabs( Li->laenge->value() * cos( PI * Li->winkel1->value() / 180 ) * cos( PI * Li->winkel2->value() / 180 ) );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dx = - dz * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dz * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dy = - dz * tan( PI * Li->winkel2->value() / 180 );
							else
								dy = dz * tan( PI * Li->winkel2->value() / 180 );
							
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dz = -dz;
							
							break;
						
						case 1:
							if( Li->winkel1->value() == 0 | Li->winkel1->value() == 180 | Li->winkel1->value() == -180 )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 1), kann es keinen X-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel1->value() > 0 )
								dx = Li->laenge->value();
							else
								dx = - Li->laenge->value();
							
							dz = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel1->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel2->value() ) > 90 ) ) )
								dy = - dz * tan( PI * Li->winkel2->value() / 180 );
							else
								dy = dz * tan( PI * Li->winkel2->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dz = - dz;
							
							break;
						
						case 2:
							if( ( Li->winkel2->value() == 0 | Li->winkel2->value() == 180 | Li->winkel2->value() == -180 ) )
								{
								KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Wenn Sie die Linie nicht kippen (Winkel 2), kann es keinen Y-Wert fr die Linie geben !"), "Fehler ");
								return;
								}
							
							if( Li->winkel2->value() > 0 )
								dy = Li->laenge->value();
							else
								dy = - Li->laenge->value();
							
							dz = fabs( Li->laenge->value() * tan( PI * ( 90 - Li->winkel2->value() )  / 180 ) );
							
							if( ( ( fabs( Li->winkel1->value() ) > 90 ) ) )
								dx = - dz * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dz * tan( PI * Li->winkel1->value() / 180 );
							
							if( (fabs(Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dz= - dz;
							
							break;
						
						case 3:
							dz = Li->laenge->value();
							
							if( (fabs(Li->winkel1->value() ) > 90 ) )
								dx = - dz * tan( PI * Li->winkel1->value() / 180 );
							else
								dx = dz * tan( PI * Li->winkel1->value() / 180 );
								
							if( ( fabs( Li->winkel2->value() ) > 90 ) )
								dy = - dz * tan( PI * Li->winkel2->value() / 180 );
							else
								dy = dz * tan( PI * Li->winkel2->value() / 180 );
								
							if( ( fabs( Li->winkel1->value() ) > 90 ) | (fabs(Li->winkel2->value() ) > 90 ) )
								dz = -dz;
								
							break;
						
						
						}
					break;
				}
		}
	line = "LINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	"+QString::number( dz, 'f', 2 )+"\n";
	ncbefehlseditor->insert( line );
	Li->hide(); m = 0;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::liniex_clicked()
	{
	if( aM != -1 )
		{
		Lix->dx->setMaxValue( ma->wx );
		Lix->dx->setMinValue( -ma->wx );
		Lix->px->setMaxValue( ma->wx );
		Lix->px->setMinValue( -ma->wx );
		}
	
	lastwidget_right = showclicked( Lix,  lastwidget_right);
	
	}

void PCNCView::liniex_einuegen_clicked()
	{
	double dx;
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	if( Lix->basis->currentPageIndex() == 0 )
		dx = Lix->dx->value();
	else
		dx = Lix->px->value() - X;
	
	line = "LINIEX\n dX	"+QString::number( dx, 'f', 2 )+"\n";
	ncbefehlseditor->insert( line );
	Lix->hide(); m = 2;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::liniey_clicked()
	{
	if( aM != -1 )
		{
		Liy->dy->setMaxValue( ma->wy );
		Liy->dy->setMinValue( -ma->wy );
		Liy->py->setMaxValue( ma->wy );
		Liy->py->setMinValue( -ma->wy );
		}
	
	lastwidget_right = showclicked( Liy,  lastwidget_right);
	
	}

void PCNCView::liniey_einfuegen_clicked()
	{
	double dy;
		
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	if( Liy->basis->currentPageIndex() == 0 )
		dy = Liy->dy->value();
	else
		dy = Liy->py->value() - Y;
	
	line = "LINIEY\n dY	"+QString::number( dy, 'f', 2 )+"\n";
	ncbefehlseditor->insert( line );
	Liy->hide(); m = 4;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::liniez_clicked()
	{
	if( aM != -1 )
		{
		Liz->dz->setMaxValue( ma->wz );
		Liz->dz->setMinValue( -ma->wz );
		Liz->pz->setMaxValue( ma->wz );
		Liz->pz->setMinValue( -ma->wz );
		}
	
	lastwidget_right = showclicked( Liz,  lastwidget_right);
	
	}

void PCNCView::liniez_einfuegen_clicked()
	{
	double dz;
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	if( Liz->basis->currentPageIndex() == 0 )
		dz = Liz->dz->value();
	else
		dz = Liz->pz->value() - Z;
	
	line = "LINIEZ\n dZ	"+QString::number( dz, 'f', 2 )+"\n";
	ncbefehlseditor->insert( line );
	Liz->hide(); m = 6;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::kreis_clicked()
	{
	if( aM != -1 )
		{
		Kr->xy_1->setMaxValue( ma->s_max );
		Kr->xy_2->setMaxValue( ma->s_max );
		}
	
	lastwidget_right = showclicked( Kr,  lastwidget_right);
	
	}

void PCNCView::kreis_einfuegen_clicked()
	{
	int i;
	double r, r2, x1=0, y1=0, x2=0, y2=0, w1, w2;
	char s1=1, s2=1, kreisdev[9];
		
	for( i=0; i < 8; i++)
		{
		kreisdev[i] = '0';
		}
	kreisdev[i] = 0;
		
	switch( Kr->um_achse->currentItem() )
		{
		case 1 : kreisdev[7] = '1'; break;
		case 2 : kreisdev[6] = '1'; break;
		}
	
	r = Kr->radius->value();
	r2 = r * r;
	
	if( r == 0.00 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert fr den Radius einzugeben !"), "Fehler ");
		return;
		}
	
	if( Kr->richtung->currentItem() == 1 )
		kreisdev[5] = '1';   // rechtsrum
	
	if( Kr->kreis_basis->currentPageIndex() == 1 )
		{   // Winkelvorgabe
		
		w1 = ( 6.2832 * Kr->winkel1->value() ) / 360.00;
		
		y1 = r * sin( w1 );
		x1 = sqrt( r2 - ( y1 * y1 ) );
		
		if( x1 < y1 )
			{
			x1 = r * cos( w1 );
			y1 = sqrt( r2 - ( x1 * x1 ) );
			}
		
		w2 = ( 6.2832 * Kr->winkel2->value() ) / 360.00;
			
		y2 = r * sin( w2 );
		x2 = sqrt( r2 - ( y2 * y2 ) );
		
		if( x2 < y2 )
			{
			x2 = r * cos( w2 );
			y2 = sqrt( r2 - ( x2 * x2 ) );
			}
		
		w1 = Kr->winkel1->value();
		w2 = Kr->winkel2->value();
		
		if( 0.00 <= w1 && w1 < 90.00 )
			s1 = 1;
		else
		if( 90.00 <= w1 && w1 < 180.00 )
			s1 = 2;
		else
		if( 180.00 <= w1 && w1 < 270.00 )
			s1 = 3;
		else
		if( 270.00 <= w1 && w1 < 360.00 )
			s1 = 4;
		
		if( 0.00 <= w2 && w2 < 90.00 )
			s2 = 1;
		else
		if( 90.00 <= w2 && w2 < 180.00 )
			s2 = 2;
		else
		if( 180.00 <= w2 && w2 < 270.00 )
			s2 = 3;
		else
		if( 270.00 <= w2 && w2 <= 360.00 )
			s2 = 4;
		}
		
	else
		{   // Punktvorgabe
		
		s1 = char( round( ( Kr->sektor_1->currentItem() + 1.1 ) / 2 ) );
		s2 = char( round( ( Kr->sektor_2->currentItem() + 1.1 ) / 2 ) );
		
		switch( Kr->sektor_1->currentItem() )
			{
			case 0: case 2: case 4: case 6:   // X
				x1 = Kr->xy_1->value();
				if( x1 > r )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 1. X-Wert liegt auserhalb des Kreises !"), "Fehler ");
					return;
					}
				
				y1 = sqrt( r2 - ( x1 * x1 ) );
				break;
			
			case 1: case 3: case 5: case 7:   // Y
				y1 = Kr->xy_1->value();
				if( y1 > r )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 1. Y-Wert liegt auserhalb des Kreises !"), "Fehler ");
					return;
					}
				x1 = sqrt( r2 - ( y1 * y1 ) );
				break;
			}
		
		switch( Kr->sektor_2->currentItem() )
			{
			case 0: case 2: case 4: case 6:   // X
				x2 = Kr->xy_2->value();
				if( x2 > r )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr  2. X-Wert liegt auserhalb des Kreises !"), "Fehler ");
					return;
					}
				y2 = sqrt( r2 - ( x2 * x2 ) );
				break;
			
			case 1: case 3: case 5: case 7:  // Y
				y2 = Kr->xy_2->value();
				if( y2 > r )
					{
					KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 2. Y-Wert liegt auserhalb des Kreises !"), "Fehler ");
					return;
					}
				x2 = sqrt( r2 - ( y2 * y2 ) );
				break;
			}
		}
	
	line = "KREIS\n";
	line += GetCircle::getCircle( mfaktor, r, &kreisdev[0], Kr->praezision->value(), s1, x1, y1, s2, x2, y2, &c1 );
	ncbefehlseditor->insert( line );
	Kr->hide(); m = 8;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::vorschub_clicked()
	{
	Vrs->vs_max_x->setValue( ma->vx );
	Vrs->vs_max_y->setValue( ma->vy );
	Vrs->vs_max_z->setValue( ma->vz );
	Vrs->vs_schlichten->setValue( ma->vs_schlichten );
	Vrs->vs_eintauchen->setValue( ma->vs_eintauchen );
	
	lastwidget_right = showclicked( Vrs,  lastwidget_right);
	
	}

void PCNCView::vs_normal_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_normal->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_eintauchen_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_eintauchen->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_bohren_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_bohren->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_schlichten_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_schlichten->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_max_x_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_max_x->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_max_y_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_max_y->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::vs_max_z_einfuegen_clicked()
	{
	line = "VORSCHUB\n [mm/min]	"+QString::number( Vrs->vs_max_z->value() )+"\n";
	ncbefehlseditor->insert( line );
	Vrs->hide(); m = 10;
	ncbefehlseditor->setFocus();
	}

void PCNCView::wechsel_clicked()
	{
	if( aM == -1 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben noch keine Maschine ausgewhlt !\nDie Werkzeuge sind somit unbekannt\nWhlen Sie mit dem Knopf Anfang eine Maschine aus !"), "Fehler ");
		return;
		}
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );
		int i = 0;
		QCString name;
		Ww->werkzeug1->clear();
		Ww->werkzeug2->clear();
		while ( ! str1.atEnd()  )
			{
			str1 >> name;
			Ww->werkzeug1->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			Ww->werkzeug2->insertItem( "("+QString::number( i )+") "+QString(name), -1 );
			i++;
			f1.at( i * 79 );
			}
		f1.close();
		if ( aW != -1)
			Ww->werkzeug1->setCurrentItem( aW );
		}
	
	lastwidget_right = showclicked( Ww,  lastwidget_right);
	
	}

void PCNCView::wechsel_einfuegen_clicked()
	{
	double dx, dy, dz, da, dif1, dif2;
	QCString name;
	Q_INT16 verwendung;
		
	if( Ww->basis->currentPageIndex() == 0 )
		{   // Relativ
		dx = Ww->dx->value();
		dy = Ww->dy->value();
		dz = Ww->dz->value();
		}
	else
		{   // Absolut
		dx = Ww->px->value() - X;
		dy = Ww->px->value() - Y;
		dz = Ww->px->value() - Z;
		}
	
	line = "WERKZEUGWECHSEL\n zu [Nummer]	"+QString::number( Ww->werkzeug2->currentItem() )+"\n";
	
	QFile f1( werkzeug );
	if ( f1.open( IO_ReadOnly ) ) 
		{
		QDataStream str1( &f1 );		
		f1.at( Ww->werkzeug1->currentItem() * 79 );
		str1 >> name >> verwendung >> da >> da >> da >> da >> dif1;
		f1.at( Ww->werkzeug2->currentItem() * 79 );
		str1 >> name >> verwendung >> da >> da >> da >> da >> dif2;
		f1.close();
		}
	
	da = ( dif2 - dif1 ) / (100.0 * mfaktor);
	
	if( da >= 0.00 )
		{
		line += "LINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	"+QString::number( dz+da, 'f', 2 )+"\nWARTE\n";
		line += "LINIE\n dX	"+QString::number( -dx, 'f', 2 )+"\n dY	"+QString::number( -dy, 'f', 2 )+"\n dZ	"+QString::number( -dz, 'f', 2 )+"\n";
		}
	else
		{
		line += "LINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	"+QString::number( dz, 'f', 2 )+"\nWARTE\n";
		line += "LINIE\n dX	"+QString::number( -dx, 'f', 2 )+"\n dY	"+QString::number( -dy, 'f', 2 )+"\n dZ	"+QString::number( -dz+da, 'f', 2 )+"\n";
		}
		
	ncbefehlseditor->insert( line );
	aW = Ww->werkzeug2->currentItem();
	m = 12; Ww->hide();
	ncbefehlseditor->setFocus();
	}

void PCNCView::werkzeugwechsel( int i )
	{
	double diffz1, diffz2, d, da, dx, dy, dz;
	QCString name;
	Q_INT16 verwendung;

	if( aW != i )
		{
		QFile f1( werkzeug );
		if ( f1.open( IO_ReadOnly ) ) 
			{
			QDataStream str1( &f1 );
			f1.at( aW * 79 );
			str1 >> name >> verwendung >> d >> d >> d >> d >> diffz1;
			f1.at( i * 79 );
			str1 >> name >> verwendung >> d >> d >> d >> d >> diffz2;
			f1.close();
			}
		
		w->exec();
		
		line = "WERKZEUGWECHSEL\n zu [Nummer]	"+QString::number( i )+"\nVORSCHUB\n [mm/min]	"+QString::number( ma->vz )+"\n";
		
		dx = w->d_x( X );
		dy = w->d_y( Y );
		dz = w->d_z( Z );
		da = ( diffz2 - diffz1 ) / (100.0 * mfaktor);
		
		if( da >= 0.00 )
			{
			line += "LINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	"+QString::number( dz+da, 'f', 2 )+"\nWARTE\n";
			line += "LINIE\n dX	"+QString::number( -dx, 'f', 2 )+"\n dY	"+QString::number( -dy, 'f', 2 )+"\n dZ	"+QString::number( -dz, 'f', 2 )+"\n";
			}
		else
			{
			line += "LINIE\n dX	"+QString::number( dx, 'f', 2 )+"\n dY	"+QString::number( dy, 'f', 2 )+"\n dZ	"+QString::number( dz, 'f', 2 )+"\nWARTE\n";
			line += "LINIE\n dX	"+QString::number( -dx, 'f', 2 )+"\n dY	"+QString::number( -dy, 'f', 2 )+"\n dZ	"+QString::number( -dz+da, 'f', 2 )+"\n";
			}
		
		ncbefehlseditor->insert( line );
		ncbefehlseditor->setFocus();
		aW = i;
		}
	}

void PCNCView::schleife_clicked()
	{
	lastwidget_right = showclicked( S,  lastwidget_right);
	}

void PCNCView::alles_einfuegen_clicked()
	{
	
	
	line="SCHLEIFE{\n\n}SCHLEIFE\n anzahl	"+QString::number( S->anzahl->value() )+"\n";
	ncbefehlseditor->insert( line );
	
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	
	ncbefehlseditor->setCursorPosition( zeile -3, 0 );
	ncbefehlseditor->setFocus();
	m = 14; S->hide();
	}

void PCNCView::spiel_clicked()
	{
	Sp->x->setValue( ma->spx );
	Sp->y->setValue( ma->spy );
	Sp->z->setValue( ma->spz );
	
	lastwidget_right = showclicked( Sp,  lastwidget_right);
	}

void PCNCView::spiel_einfuegen_clicked()
	{
	
	line="SPIEL\n X	"+QString::number( Sp->x->value() )+"\n Y	"+QString::number( Sp->y->value() )+"\n Z	"+QString::number( Sp->z->value() )+"\n";
	ncbefehlseditor->insert( line );
	
	ncbefehlseditor->setFocus();
	m = 16; Sp->hide();
	}

void PCNCView::ellipse_clicked()
	{
	if( aM != -1 )
		{
		El->w1->setMaxValue( ma->s_max );
		El->w2->setMaxValue( ma->s_max );
		}
	
	lastwidget_right = showclicked( El,  lastwidget_right);
	}

void PCNCView::pause_clicked()
	{
	line = "WARTE\n- auf Return/Enter\n";
	ncbefehlseditor->insert( line );
	}

void PCNCView::memo_clicked()
	{
	lastwidget_right = showclicked( M,  lastwidget_right);
	}

void PCNCView::beschleunigung_clicked()
	{
	lastwidget_right = showclicked( B,  lastwidget_right);
	}

void PCNCView::f_linear_clicked()
	{
	if( aM != -1 )
		FL->delta->setMaxValue( ma->s_max );
	
	lastwidget_F = showclicked( FL,  lastwidget_F);
	}

void PCNCView::f_linear_einfuegen_clicked()
	{
	char status_L[8];
	int i;
	
	for( i=0; i < 8; i++)
		status_L[i] = '0';
	
	status_L[i] = 0;
	
	switch ( FL->status->currentPageIndex() )
		{
		case 0:
			switch ( FL->achsen_l->currentItem() )
				{
				case 0: status_L[5] = '1'; break;
				case 1: status_L[4] = '1'; break;
				case 2: status_L[3] = '1'; break;
				case 3: status_L[5] = '1'; status_L[4] = '1'; break;
				case 4: status_L[5] = '1'; status_L[3] = '1'; break;
				case 5: status_L[4] = '1'; status_L[3] = '1'; break;
				case 6: status_L[5] = '1'; status_L[4] = '1'; status_L[3] = '1'; break;
				}
			
			switch ( FL->op_l->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 1:
			status_L[5] = '1';
			switch ( FL->op_lx->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 2:
			status_L[4] = '1';
			switch ( FL->op_ly->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 3:
			status_L[3] = '1';
			switch ( FL->op_lz->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 4:
			if( FL->op_lz->currentItem() == 0 )
				status_L[2] = '1';
			break;
		
		case 5:
			switch ( FL->part_el->currentItem() )
				{
				case 0:
					 if( FL->op_el->currentItem() == 0 )
						{
						status_L[0] = '1';
						status_L[2] = '1';
						}
					else
						status_L[0] = '1';
					break;
				
				case 1:
					 if( FL->op_el->currentItem() == 0 )
						{
						status_L[1] = '1';
						status_L[2] = '1';
						}
					else
						status_L[2] = '1';
					break;
				
				case 2:
					if( FL->op_el->currentItem() == 0 )
						status_L[2] = '1';
					break;
				}
		 break;
		}
	line = "DELTALINEAR\n status	";
	line += status_L;
	line += "\n delta	"+QString::number( FL->delta->value(), 'f', 2 )+"\n";
	ncbefehlseditor->insert( line );
	FL->hide(); f = 0;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::f_kreis_clicked()
	{
	if( aM != -1 )
		{
		FK->xy_1->setMaxValue( ma->s_max );
		FK->xy_2->setMaxValue( ma->s_max );
		}
	
	lastwidget_F = showclicked( FK,  lastwidget_F);
	}

void PCNCView::kreisfunktion_einfuegen_clicked()
	{
	int i;
	double r, r2, x1=0, y1=0, x2=0, y2=0;
	char s1=1, s2=1, kreisdev[9];
		
	for( i=0; i < 8; i++)
		{
		kreisdev[i] = '0';
		}
	kreisdev[i] = 0;

	r = FK->radius->value();
	r2 = r * r;
	
	if( r == 0.00 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert fr den Radius einzugeben !"), "Fehler ");
		return;
		}
	
	s1 = char( round( ( FK->sektor_1->currentItem() + 1 ) / 2 ) );
	s2 = char( round( ( FK->sektor_2->currentItem() + 1 ) / 2 ) );
	
	switch( FK->sektor_1->currentItem() )
		{
		case 0: case 2: case 4: case 6:   // X
			x1 = FK->xy_1->value();
			if( x1 > r )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 1. X-Wert liegt auserhalb des Kreises !"), "Fehler ");
				return;
				}
			
			y1 = sqrt( r2 - ( x1 * x1 ) );
			break;
		
		case 1: case 3: case 5: case 7:   // Y
			y1 = FK->xy_1->value();
			if( y1 > r )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 1. Y-Wert liegt auserhalb des Kreises !"), "Fehler ");
				return;
				}
			x1 = sqrt( r2 - ( y1 * y1 ) );
			break;
		}
	
	switch( FK->sektor_2->currentItem() )
		{
		case 0: case 2: case 4: case 6:   // X
			x2 = FK->xy_2->value();
			if( x2 > r )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr  2. X-Wert liegt auserhalb des Kreises !"), "Fehler ");
				return;
				}
			y2 = sqrt( r2 - ( x2 * x2 ) );
			break;
		
		case 1: case 3: case 5: case 7:  // Y
			y2 = FK->xy_2->value();
			if( y2 > r )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Ihr 2. Y-Wert liegt auserhalb des Kreises !"), "Fehler ");
				return;
				}
			x2 = sqrt( r2 - ( y2 * y2 ) );
			break;
		}

	if( FK->precision->value() == 0 )
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, QString("Sie haben fr Przision keinen Wert angeben.\n Weiter ?"), "Warnung ") )
			return;
	
	line = "KREISFUNKTION\n";
	line += GetCircle::getCircle( mfaktor, r, &kreisdev[0], FK->precision->value(), s1, x1, y1, s2, x2, y2, &c1 );
	ncbefehlseditor->insert( line );
	FK->hide(); f = 2;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	ncbefehlseditor->setFocus();
	}

void PCNCView::deltakreis_einfuegen_clicked()
	{
	int i;
	char status_L[9];
	
	for( i=0; i < 8; i++)
		status_L[i] = '0';
	
	status_L[i] = 0;
	
	switch ( FK->status->currentPageIndex() )
		{
		case 0:
			switch ( FK->achsen_l->currentItem() )
				{
				case 0: status_L[5] = '1'; break;
				case 1: status_L[4] = '1'; break;
				case 2: status_L[3] = '1'; break;
				case 3: status_L[5] = '1'; status_L[4] = '1'; break;
				case 4: status_L[5] = '1'; status_L[3] = '1'; break;
				case 5: status_L[4] = '1'; status_L[3] = '1'; break;
				case 6: status_L[5] = '1'; status_L[4] = '1'; status_L[3] = '1'; break;
				}
			
			switch ( FK->op_l->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 1:
			status_L[5] = '1';
			switch ( FK->op_lx->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 2:
			status_L[4] = '1';
			switch ( FK->op_ly->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 3:
			status_L[3] = '1';
			switch ( FK->op_lz->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 4:
			if( FK->op_lz->currentItem() == 0 )
				status_L[2] = '1';
			break;
		
		case 5:
			switch ( FK->part_el->currentItem() )
				{
				case 0:
					 if( FK->op_el->currentItem() == 0 )
						{
						status_L[0] = '1';
						status_L[2] = '1';
						}
					else
						status_L[0] = '1';
					break;
				
				case 1:
					 if( FK->op_el->currentItem() == 0 )
						{
						status_L[1] = '1';
						status_L[2] = '1';
						}
					else
						status_L[2] = '1';
					break;
				
				case 2:
					if( FK->op_el->currentItem() == 0 )
						status_L[2] = '1';
					break;
				}
		 break;
		}
	line = "DELTAKREIS\n status	";
	line += status_L;
	line += "\n";
	ncbefehlseditor->insert( line );
	FK->hide(); f = 2;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::f_ellipse_clicked()
	{
	if( aM != -1 )
		{
		FE->xy_1->setMaxValue( ma->s_max );
		FE->xy_2->setMaxValue( ma->s_max );
		}
	
	lastwidget_F = showclicked( FE,  lastwidget_F);
	}

void PCNCView::ellipsfunktion_einfuegen_clicked()
	{
	int  i, zeile, spalte;
	char s1, s2, eldef[9];
	double x1=0, y1=0, x2=0, y2=0, r1, r2;
	
	
	r1 = FE->r1->value();
	r2 = FE->r2->value();
	
	if( r1 > r2 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Innenradius mu natrlich kleiner als der Auenradius sein !"), "Fehler ");
		return;
		}
	if( r1 == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert fr den Innenradius anzugeben !"), "Fehler ");
		return;
		}
	
	for( i=0; i < 8; i++)
		{
		eldef[i] = '0';
		}
	eldef[i] = 0;
	
	eldef[5] = '0';   // left around
	
	s1 = char( round( (FE->sektor_1->currentItem() + 1.1 ) / 2 ) );
	s2 = char( round( (FE->sektor_2->currentItem() + 1.1 ) / 2 ) );
	
	// um Z lngs X   -> x, y
	eldef[6] = '0';
	eldef[7] = '0';
	switch( FE->sektor_1->currentItem() )
		{
		case 0: case 2: case 4: case 6:
			x1 = FE->xy_1->value();
			if( x1 > r2 )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert X liegt auerhalb der Ellipse !"), "Fehler ");
				return;
				}
			y1 = r1 / r2 * sqrt( r2 * r2 - x1 * x1 );
			break;
		case 1: case 3: case 5: case 7:
			y1 = FE->xy_1->value();
			if( y1 > r1 )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Y liegt auerhalb der Ellipse !"), "Fehler ");
				return;
				}
			x1 = sqrt( r2 * r2 - (y1 * r2 / r1) * (y1 * r2 / r1) );
		}
	
	switch( FE->sektor_2->currentItem() )
		{
		case 0: case 2: case 4: case 6:
			x2 = FE->xy_2->value();
			if( x2 > r2 )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt X liegt auerhalb der Ellipse !"), "Fehler ");
				return;
				}
			y2 = r1 / r2 * sqrt( r2 * r2 - x2 * x2 );
			break;
		case 1: case 3: case 5: case 7:
			y2 = FE->xy_2->value();
			if( y2 > r1 )
				{
				KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Y liegt auerhalb der Ellipse !"), "Fehler ");
				return;
				}
			x2 = sqrt( r2 * r2 - (y2 * r2 / r1) * (y2 * r2 / r1) );
		}
	
	if( FE->precision->value() == 0 )
		if( KMessageBox::No == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, QString("Sie haben fr Przision keinen Wert angeben.\n Weiter ?"), "Warnung ") )
			return;
	
	line = "ELLIPSENFUNKTION\n";
	line += getEllips( mfaktor, r1, r2, &eldef[0], FE->precision->value(), s1, x1, y1, s2, x2, y2, &e1);
	ncbefehlseditor->insert( line );
	FE->hide(); f=4;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	
	}

void PCNCView::deltaellips_einfuegen_clicked()
	{
	char status_L[8];
	int i;
	
	for( i=0; i < 8; i++)
		status_L[i] = '0';
	
	status_L[i] = 0;
	
	switch ( FE->status->currentPageIndex() )
		{
		case 0:
			switch ( FE->achsen_l->currentItem() )
				{
				case 0: status_L[5] = '1'; break;
				case 1: status_L[4] = '1'; break;
				case 2: status_L[3] = '1'; break;
				case 3: status_L[5] = '1'; status_L[4] = '1'; break;
				case 4: status_L[5] = '1'; status_L[3] = '1'; break;
				case 5: status_L[4] = '1'; status_L[3] = '1'; break;
				case 6: status_L[5] = '1'; status_L[4] = '1'; status_L[3] = '1'; break;
				}
			
			switch ( FE->op_l->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 1:
			status_L[5] = '1';
			switch ( FE->op_lx->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 2:
			status_L[4] = '1';
			switch ( FE->op_ly->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 3:
			status_L[3] = '1';
			switch ( FE->op_lz->currentItem() )
				{
				case 1: status_L[7] = '1'; break;
				case 2: status_L[6] = '1'; break;
				case 3: status_L[7] = '1'; status_L[6] = '1'; break;
				}
			break;
		
		case 4:
			if( FE->op_lz->currentItem() == 0 )
				status_L[2] = '1';
			break;
		
		case 5:
			switch ( FE->part_el->currentItem() )
				{
				case 0:
					 if( FE->op_el->currentItem() == 0 )
						{
						status_L[0] = '1';
						status_L[2] = '1';
						}
					else
						status_L[0] = '1';
					break;
				
				case 1:
					 if( FE->op_el->currentItem() == 0 )
						{
						status_L[1] = '1';
						status_L[2] = '1';
						}
					else
						status_L[2] = '1';
					break;
				
				case 2:
					if( FE->op_el->currentItem() == 0 )
						status_L[2] = '1';
					break;
				}
		 break;
		}
	line = "DELTAELLIPS\n status	";
	line += status_L;
	line += "\n";
	ncbefehlseditor->insert( line );
	FE->hide(); f = 4;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::ellipse_einfuegen_clicked()
	{
	int  i, zeile, spalte;
	char s1, s2, eldef[9];
	double x1=0, y1=0, x2=0, y2=0, r1, r2;
	
	
	r1 = El->r1->value();
	r2 = El->r2->value();
	
	if( r1 > r2 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Innenradius mu natrlich kleiner als der Auenradius sein !"), "Fehler ");
		return;
		}
	if( r1 == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert fr den Innenradius anzugeben !"), "Fehler ");
		return;
		}
	
	for( i=0; i < 8; i++)
		{
		eldef[i] = '0';
		}
	eldef[i] = 0;
	
	if( El->richtung->currentItem() == 0 )
		eldef[4] = '0';   // left around
	else
		eldef[4] = '1';   // right around
	
	s1 = char( round( (El->sektor_1->currentItem() + 1.1 ) / 2 ) );
	s2 = char( round( (El->sektor_2->currentItem() + 1.1 ) / 2 ) );
	
	switch( El->ausrichtung->currentItem() )
		{
		case 0: // um Z lngs X   -> x, y
			eldef[5] = '0';
			eldef[6] = '0';
			eldef[7] = '0';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2 * r2 - x1 * x1 );
					break;
				case 1: case 3: case 5: case 7:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 - (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				}
			
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2 * r2 - x2 * x2 );
					break;
				case 1: case 3: case 5: case 7:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 - (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				}
			break;
		
		case 1: // um Z lngs Y   -> y, x
			eldef[5] = '0';
			eldef[6] = '0';
			eldef[7] = '1';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 -  (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2 * r2 - x1 * x1 );
					break;
				}
			
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 -  (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2 * r2 - x2 * x2 );
					break;
				}
			break;
		
		case 2: // um Y lngs X   -> x, z
			eldef[5] = '0';
			eldef[6] = '1';
			eldef[7] = '0';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2*r2 - x1*x1 );
					break;
				case 1: case 3: case 5: case 7:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 -  (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				}
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2*r2 - x2*x2 );
					break;
				case 1: case 3: case 5: case 7:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 -  (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				}
			break;
		
		case 3: // um Y lngs Z   -> z, x
			eldef[5] = '0';
			eldef[6] = '1';
			eldef[7] = '1';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 -  (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2*r2 - x1*x1 );
					break;
				}
			
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt X liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 -  (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2 * r2 - x2 * x2 );
					break;
				}
			break;
		
		case 4: // um X lngs Y   -> y, z
			eldef[5] = '1';
			eldef[6] = '0';
			eldef[7] = '0';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2*r2 - x1*x1 );
					break;
				case 1: case 3: case 5: case 7:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 -  (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				}
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2*r2 - x2*x2 );
					break;
				case 1: case 3: case 5: case 7:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 -  (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				}
			break;
		
		case 5: // um X lngs Z   -> z, y
			eldef[5] = '1';
			eldef[6] = '0';
			eldef[7] = '1';
			switch( El->sektor_1->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y1 = El->w1->value();
					if( y1 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x1 = sqrt( r2 * r2 -  (y1 * r2 / r1) * (y1 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x1 = El->w1->value();
					if( x1 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Startwert Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y1 = r1 / r2 * sqrt( r2*r2 - x1 * x1 );
					break;
				}
			
			switch( El->sektor_2->currentItem() )
				{
				case 0: case 2: case 4: case 6:
					y2 = El->w2->value();
					if( y2 > r1 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Y liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					x2 = sqrt( r2 * r2 -  (y2 * r2 / r1) * (y2 * r2 / r1) );
					break;
				case 1: case 3: case 5: case 7:
					x2 = El->w2->value();
					if( x2 > r2 )
						{
						KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Der Endpunkt Z liegt auerhalb der Ellipse !"), "Fehler ");
						return;
						}
					y2 = r1 / r2 * sqrt( r2 * r2 - x2 * x2 );
					break;
				}
			break;
		}
	
	line = "ELLIPSE\n";
	line += getEllips( mfaktor, r1, r2, &eldef[0], El->praezision->value(), s1, x1, y1, s2, x2, y2, &e1);
	ncbefehlseditor->insert( line );
	El->hide(); m=18;
	ncbefehlseditor->getCursorPosition( &zeile, &spalte );
	getcurrentpos( ncbefehlseditor, zeile, &X, &Y, &Z);
	emit signalChangeStatusbar(" Position nach dem letzten eingesetzten Objekt :   X = "+QString::number( X, 'f', 2 )+"   Y = "+QString::number( Y, 'f', 2 )+"   Z = "+QString::number( Z, 'f', 2 ) );
	ncbefehlseditor->setFocus();
	}

void PCNCView::memo_einfuegen_clicked()
	{
	if( M->memo->length() == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Text einzugeben !"), "Fehler ");
		return;
		}
	line = "MEMO\n lnge	"+QString::number( M->memo->length() )+"\n";
	ncbefehlseditor->insert( line + M->memo->text() + "\n" );
	M->hide();
	ncbefehlseditor->setFocus();
	}


void PCNCView::beschleunigung_einfuegen_clicked()
	{
	if( B->a->value() == 0 )
		{
		KMessageBox::messageBox ( 0, KMessageBox::Information, QString("Sie haben vergessen einen Wert einzugeben !"), "Fehler ");
		return;
		}
	line = "BESCHLEUNIGUNG\n [mm/min]	"+QString::number( B->a->value() )+"\n";
	ncbefehlseditor->insert( line );
	B->hide();
	ncbefehlseditor->setFocus();
	}




#include "pcncview.moc"

